@@ -4,7 +4,7 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2014-2018, Will Godfrey
+    Copyright 2014-2019, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -22,7 +22,6 @@
 
     This file is derivative of ZynAddSubFX original code.
 
-    Modified July 2018
 */
 
 #include "Misc/SynthEngine.h"
@@ -57,6 +56,7 @@ DynamicFilter::DynamicFilter(bool insertion_, float *efxoutl_, float *efxoutr_,
     setvolume(110);
     setpreset(Ppreset);
     changepar(1, 64); // pan
+    Pchanged = false;
     cleanup();
 }
 
@@ -283,11 +283,17 @@ if (npreset < 0xf)
         if ((insertion == 0) && (param == 0))
             changepar(0, presets[preset][0] * 0.5f);
     }
+    Pchanged = false;
 }
 
 
 void DynamicFilter::changepar(int npar, unsigned char value)
 {
+    if (npar == -1)
+    {
+        Pchanged = (value != 0);
+        return;
+    }
     switch (npar)
     {
         case 0:
@@ -336,6 +342,7 @@ void DynamicFilter::changepar(int npar, unsigned char value)
             setampsns(Pampsns);
             break;
     }
+    Pchanged = true;
 }
 
 
@@ -343,6 +350,7 @@ unsigned char DynamicFilter::getpar(int npar)
 {
     switch (npar)
     {
+        case -1: return Pchanged;
         case 0:  return Pvolume;
         case 1:  return Ppanning;
         case 2:  return lfo.Pfreq;
@@ -361,7 +369,7 @@ unsigned char DynamicFilter::getpar(int npar)
 
 float Dynamlimit::getlimits(CommandBlock *getData)
 {
-    int value = getData->data.value;
+    int value = getData->data.value.F;
     int control = getData->data.control;
     int request = getData->data.type & 3; // clear upper bits
     int npart = getData->data.part;
@@ -370,12 +378,12 @@ float Dynamlimit::getlimits(CommandBlock *getData)
     int max = 127;
 
     int def = presets[presetNum][control];
-    bool canLearn = true;
-    bool isInteger = true;
+    unsigned char canLearn = TOPLEVEL::type::Learnable;
+    unsigned char isInteger = TOPLEVEL::type::Integer;
     switch (control)
     {
         case 0:
-            if (npart != TOPLEVEL::section::systemEffects) // system effects
+            if (npart == TOPLEVEL::section::systemEffects) // system effects
                 def /= 2;
             break;
         case 1:
@@ -386,7 +394,7 @@ float Dynamlimit::getlimits(CommandBlock *getData)
             break;
         case 4:
             max = 1;
-            canLearn = false;
+            canLearn = 0;
             break;
         case 5:
             break;
@@ -396,13 +404,13 @@ float Dynamlimit::getlimits(CommandBlock *getData)
             break;
         case 8:
             max = 1;
-            canLearn = false;
+            canLearn = 0;
             break;
         case 9:
             break;
         case 16:
             max = 4;
-            canLearn = false;
+            canLearn = 0;
             break;
         default:
             getData->data.type |= TOPLEVEL::type::Error;
@@ -428,7 +436,7 @@ float Dynamlimit::getlimits(CommandBlock *getData)
             value = def;
             break;
     }
-    getData->data.type |= (canLearn * 64 + isInteger * 128);
+    getData->data.type |= (canLearn + isInteger);
     return float(value);
 }
 
