@@ -1,10 +1,10 @@
 /*
-    EQ.cpp - EQ effect
+    EQ.cpp - Equalizer effect
 
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2010, Alan Calvert
-    Copyright 2018, Will Godfrey
+    Copyright 2018-2019, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -22,11 +22,14 @@
 
     This file is derivative of ZynAddSubFX original code.
 
-    Modified July 2018
 */
 
 #include "Misc/SynthEngine.h"
 #include "Effects/EQ.h"
+#include "Misc/NumericFuncs.h"
+
+using func::rap2dB;
+
 
 EQ::EQ(bool insertion_, float *efxoutl_, float *efxoutr_, SynthEngine *_synth) :
     Effect(insertion_, efxoutl_, efxoutr_, NULL, 0),
@@ -46,6 +49,7 @@ EQ::EQ(bool insertion_, float *efxoutl_, float *efxoutr_, SynthEngine *_synth) :
     setvolume(50);
     Pband = 0;
     setpreset(Ppreset);
+    Pchanged = false;
     cleanup();
 }
 
@@ -138,11 +142,17 @@ void EQ::setpreset(unsigned char npreset)
     for (int n = 0; n < PRESET_SIZE; ++n)
         changepar(n, presets[npreset][n]);
     Ppreset = npreset;
+    Pchanged = true;
 }
 
 
 void EQ::changepar(int npar, unsigned char value)
 {
+    if (npar == -1)
+    {
+        Pchanged = (value != 0);
+        return;
+    }
     switch (npar)
     {
         case 0:
@@ -157,7 +167,7 @@ void EQ::changepar(int npar, unsigned char value)
     int nb = (npar - 10) / 5; // number of the band (filter)
     if (nb >= MAX_EQ_BANDS)
         return;
-    int bp = npar % 5; // band paramenter
+    int bp = npar % 5; // band parameter
 
     float tmp;
     switch (bp)
@@ -199,6 +209,7 @@ void EQ::changepar(int npar, unsigned char value)
             filter[nb].r->setstages(value);
             break;
     }
+    Pchanged = true;
 }
 
 
@@ -206,6 +217,7 @@ unsigned char EQ::getpar(int npar)
 {
     switch (npar)
     {
+        case -1: return Pchanged;
         case 0:
             return Pvolume;
             break;
@@ -218,7 +230,7 @@ unsigned char EQ::getpar(int npar)
     int nb = (npar - 10) / 5; // number of the band (filter)
     if (nb >= MAX_EQ_BANDS)
         return 0;
-    int bp = npar % 5; // band paramenter
+    int bp = npar % 5; // band parameter
     switch (bp)
     {
         case 0:
@@ -261,15 +273,15 @@ float EQ::getfreqresponse(float freq)
 
 float EQlimit::getlimits(CommandBlock *getData)
 {
-    int value = getData->data.value;
+    int value = getData->data.value.F;
     int control = getData->data.control;
     int request = getData->data.type & 3; // clear upper bits
 
     int min = 0;
     int max = 127;
     int def = 0;
-    bool canLearn = true;
-    bool isInteger = true;
+    unsigned char canLearn = TOPLEVEL::type::Learnable;
+    unsigned char isInteger = TOPLEVEL::type::Integer;
 
     switch (control)
     {
@@ -278,11 +290,11 @@ float EQlimit::getlimits(CommandBlock *getData)
             break;
         case 1:
             max = 7;
-            canLearn = false;
+            canLearn = 0;
             break;
         case 10:
             max = 9;
-            canLearn = false;
+            canLearn = 0;
             break;
         case 11:
         case 12:
@@ -291,7 +303,7 @@ float EQlimit::getlimits(CommandBlock *getData)
             break;
         case 14:
             max = 4;
-            canLearn = false;
+            canLearn = 0;
             break;
         default:
             getData->data.type |= TOPLEVEL::type::Error;
@@ -317,7 +329,7 @@ float EQlimit::getlimits(CommandBlock *getData)
             value = def;
             break;
     }
-    getData->data.type |= (canLearn * 64 + isInteger * 128);
+    getData->data.type |= (canLearn + isInteger);
     return float(value);
 }
 
