@@ -4,7 +4,7 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2009 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2017-2018, Will Godfrey
+    Copyright 2017-2019, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -22,7 +22,6 @@
 
     This file is derivative of ZynAddSubFX original code.
 
-    Modified February 2018
 */
 
 #include <fftw3.h>
@@ -152,24 +151,15 @@ unsigned char EffectMgr::getpreset(void)
 
 
 // Change the preset of the current effect
-void EffectMgr::changepreset_nolock(unsigned char npreset)
+void EffectMgr::changepreset(unsigned char npreset)
 {
     if (efx)
         efx->setpreset(npreset);
 }
 
 
-// Change the preset of the current effect(with thread locking)
-void EffectMgr::changepreset(unsigned char npreset)
-{
-    //synth->actionLock(lockType);
-    changepreset_nolock(npreset);
-    //synth->actionLock(unlockType);
-}
-
-
 // Change a parameter of the current effect
-void EffectMgr::seteffectpar_nolock(int npar, unsigned char value)
+void EffectMgr::seteffectpar(int npar, unsigned char value)
 {
     if (!efx)
         return;
@@ -177,15 +167,6 @@ void EffectMgr::seteffectpar_nolock(int npar, unsigned char value)
 }
 
 
-// Change a parameter of the current effect (with thread locking)
-void EffectMgr::seteffectpar(int npar, unsigned char value)
-{
-    //synth->actionLock(lockType);
-    seteffectpar_nolock(npar, value);
-    //synth->actionLock(unlockType);
-}
-
-
 // Get a parameter of the current effect
 unsigned char EffectMgr::geteffectpar(int npar)
 {
@@ -318,19 +299,27 @@ void EffectMgr::getfromXML(XMLwrapper *xml)
     changeeffect(xml->getpar127("type", geteffect()));
     if (!efx || !geteffect())
         return;
-    efx->Ppreset = xml->getpar127("preset", efx->Ppreset);
+    changepreset(xml->getpar127("preset", efx->Ppreset));
 
+    bool isChanged = false;
     if (xml->enterbranch("EFFECT_PARAMETERS"))
     {
         for (int n = 0; n < 128; ++n)
         {
-            seteffectpar_nolock(n, 0); // erase effect parameter
+            int par = geteffectpar(n); // find default
+            seteffectpar(n, 0); // erase effect parameter
             if (xml->enterbranch("par_no", n) == 0)
                 continue;
-            int par = geteffectpar(n);
-            seteffectpar_nolock(n, xml->getpar127("par", par));
+            seteffectpar(n, xml->getpar127("par", par));
+            if (par != geteffectpar(n))
+            {
+                isChanged = true;
+                //cout << "changed par " << n << endl;
+                //may use this later to ID
+            }
             xml->exitbranch();
         }
+        seteffectpar(-1, isChanged);
         if (filterpars)
         {
             if (xml->enterbranch("FILTER"))
@@ -340,6 +329,8 @@ void EffectMgr::getfromXML(XMLwrapper *xml)
             }
         }
         xml->exitbranch();
+        //if(geteffectpar(-1))
+            //cout << "Some pars changed" << endl;
     }
     cleanup();
 }
@@ -386,6 +377,8 @@ float LimitMgr::geteffectlimits(CommandBlock *getData)
             Dynamlimit dyn;
             value = dyn.getlimits(getData);
             break;
+        default:
+            value = EFFECT::type::count - EFFECT::type::none;
     }
     return value;
 }
