@@ -4,7 +4,7 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2009 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2018, Will Godfrey
+    Copyright 2018-2019, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -22,7 +22,6 @@
 
     This file is derivative of ZynAddSubFX original code.
 
-    Modified December 2018
 */
 
 #include <cmath>
@@ -110,6 +109,7 @@ Reverb::Reverb(bool insertion_, float *efxoutl_, float *efxoutr_, SynthEngine *_
         ap[i] = NULL;
     }
     setpreset(Ppreset);
+    Pchanged = false;
     cleanup(); // do not call this before the comb initialisation
 }
 
@@ -455,8 +455,7 @@ void Reverb::settype(unsigned char Ptype_)
         bandwidth = new Unison(synth->buffersize / 4 + 1, 2.0f, synth);
         bandwidth->setSize(50);
         bandwidth->setBaseFrequency(1.0f);
-#pragma message "sa schimb size-ul"
-        //the size of the unison buffer may be too small, though this has
+        //TODO the size of the unison buffer may be too small, though this has
         //not been verified yet.
         //As this cannot be resized in a RT context, a good upper bound should
         //be found
@@ -511,11 +510,17 @@ void Reverb::setpreset(unsigned char npreset)
         if (insertion && (param == 0))
             changepar(0, presets[preset][0] / 2);
     }
+    Pchanged = false;
 }
 
 
 void Reverb::changepar(int npar, unsigned char value)
 {
+    if (npar == -1)
+    {
+        Pchanged = (value != 0);
+        return;
+    }
     switch (npar)
     {
         case 0:
@@ -556,6 +561,7 @@ void Reverb::changepar(int npar, unsigned char value)
             setbandwidth(value);
             break;
     }
+    Pchanged = true;
 }
 
 
@@ -563,6 +569,7 @@ unsigned char Reverb::getpar(int npar)
 {
     switch (npar)
     {
+        case -1: return Pchanged;
         case 0:  return Pvolume;
         case 1:  return Ppanning;
         case 2:  return Ptime;
@@ -586,7 +593,7 @@ unsigned char Reverb::getpar(int npar)
 
 float Revlimit::getlimits(CommandBlock *getData)
 {
-    int value = getData->data.value;
+    int value = getData->data.value.F;
     int control = getData->data.control;
     int request = getData->data.type & 3; // clear upper bits
     int npart = getData->data.part;
@@ -595,8 +602,8 @@ float Revlimit::getlimits(CommandBlock *getData)
     int max = 127;
 
     int def = presets[presetNum][control];
-    bool canLearn = true;
-    bool isInteger = true;
+    unsigned char canLearn = TOPLEVEL::type::Learnable;
+    unsigned char isInteger = TOPLEVEL::type::Integer;
 
     switch (control)
     {
@@ -621,16 +628,16 @@ float Revlimit::getlimits(CommandBlock *getData)
             break;
         case 10:
             max = 2;
-            canLearn = false;
+            canLearn = 0;
             break;
         case 11:
-            canLearn = false;
+            canLearn = 0;
             break;
         case 12:
             break;
         case 16:
             max = 12;
-            canLearn = false;
+            canLearn = 0;
             break;
         default:
             getData->data.type |= TOPLEVEL::type::Error;
@@ -656,6 +663,6 @@ float Revlimit::getlimits(CommandBlock *getData)
             value = def;
             break;
     }
-    getData->data.type |= (canLearn * 64 + isInteger * 128);
+    getData->data.type |= (canLearn + isInteger);
     return float(value);
 }
