@@ -1,7 +1,7 @@
 /*
     MidiDecode.cpp
 
-    Copyright 2017-2018 Will Godfrey
+    Copyright 2017-2020 Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -17,7 +17,6 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    Modified December 2018
 */
 
 #include <iostream>
@@ -27,24 +26,24 @@
 #include <string>
 #include <unistd.h>
 
-using namespace std;
 
 #include "Interface/MidiDecode.h"
 #include "Interface/InterChange.h"
-#include "Misc/MiscFuncs.h"
 #include "Misc/SynthEngine.h"
+#include "Misc/FormatFuncs.h"
+
+using func::asString;
+using func::asHexString;
+
 
 MidiDecode::MidiDecode(SynthEngine *_synth) :
     synth(_synth)
 {
- //init
-
 }
 
 
 MidiDecode::~MidiDecode()
 {
-    //close
 }
 
 
@@ -107,7 +106,7 @@ void MidiDecode::setMidiController(unsigned char ch, int ctrl, int param, bool i
 {
     if (synth->getRuntime().monitorCCin)
     {
-        string ctltype;
+        std::string ctltype;
         switch (ctrl)
         {
             case MIDI::CC::null:
@@ -190,9 +189,8 @@ void MidiDecode::setMidiController(unsigned char ch, int ctrl, int param, bool i
      *
      * Some controller values are >= 640 so they will be ignored by
      * later calls, but are passed as 128+ for this call.
-     * Pitch wheel is 640 and is 14 bit. It sets bit 1 of 'category'
      */
-    if (synth->midilearn.runMidiLearn(param, ctrl & 0xff, ch, in_place | ((ctrl == 640) << 1)))
+    if (synth->midilearn.runMidiLearn(param, ctrl & 0xff, ch, in_place))
         return; // blocking while learning
 
     /*
@@ -216,21 +214,22 @@ void MidiDecode::sendMidiCC(bool inSync, unsigned char chan, int type, short int
 {
     if (inSync) // no CLI or GUI updates needed
     {
-        //cout << "CC inSync" << endl;
+        //std::cout << "CC inSync" << std::endl;
         synth->SetController(chan, type, par);
         return;
     }
 
-    //cout << "CC buffered" << endl;
+    //std::cout << "CC buffered" << std::endl;
     CommandBlock putData;
     memset(&putData, 0xff, sizeof(putData));
-    putData.data.value = par;
-    putData.data.type = 0xc8;
+    putData.data.value.F = par;
+    putData.data.type = 0xc0;
+    putData.data.source = TOPLEVEL::action::toAll;
     putData.data.control = 2;
     putData.data.part = TOPLEVEL::section::midiIn;
     putData.data.kit = chan;
     putData.data.engine = type;
-    synth->midilearn.writeMidi(&putData, sizeof(putData), false);
+    synth->midilearn.writeMidi(&putData, false);
 }
 
 /*
@@ -268,8 +267,10 @@ bool MidiDecode::nrpnDecode(unsigned char ch, int ctrl, int param, bool in_place
                     }
                     return true;
                 }
-                if (type == 0x44 && param == 0x44)
+                if (type == 0x44 && (param == 0x44 || param == 0x45))
                 {
+                    if (param == 0x45)
+                        synth->getRuntime().exitType = FORCED_EXIT;
                     synth->getRuntime().runSynth = false;
                     return true; // bye bye everyone
                 }
@@ -471,7 +472,7 @@ void MidiDecode::nrpnProcessData(unsigned char chan, int type, int par, bool in_
     */
 
     // For NRPNs midi learn must come before everything else
-    if (synth->midilearn.runMidiLearn(dHigh << 7 | par, 0x10000 | (nHigh << 7) | nLow , chan, in_place | 2))
+   if (synth->midilearn.runMidiLearn(dHigh << 7 | par, MIDI::CC::identNRPN | (nHigh << 7) | nLow , chan, in_place))
         return; // blocking while learning
 
     if (nLow < nHigh && (nHigh == 4 || nHigh == 8 ))
@@ -485,6 +486,10 @@ void MidiDecode::nrpnProcessData(unsigned char chan, int type, int par, bool in_
         return;
     }
 
+    if (nHigh == 0x60)
+        if (nrpnProcessHistory(nLow, dHigh, par, in_place))
+            return;
+
     if (nHigh != 64 && nLow < 0x7f)
     {
         synth->getRuntime().Log("Go away NRPN 0x" + asHexString(nHigh) + " " + asHexString(nLow) +" We don't know you!");
@@ -504,6 +509,42 @@ void MidiDecode::nrpnProcessData(unsigned char chan, int type, int par, bool in_
 }
 
 
+bool MidiDecode::nrpnProcessHistory(unsigned char nLow, unsigned char dHigh, unsigned char dLow, bool in_place)
+{
+    if (nLow > TOPLEVEL::XML::MLearn)
+        return false;
+    if (dLow >= MAX_HISTORY)
+        return true;// not an error but not wanted anyway!
+
+    if (in_place)
+    { // not done yet
+        return true;
+    }
+    CommandBlock putData;
+    memset(&putData, 0xff, sizeof(putData));
+    putData.data.type = TOPLEVEL::type::Integer;
+    putData.data.source = TOPLEVEL::action::fromMIDI;
+    putData.data.control = MAIN::control::loadFileFromList;
+    putData.data.part = TOPLEVEL::section::main;
+    putData.data.kit = nLow;
+    putData.data.engine = dLow;
+    if (nLow == TOPLEVEL::XML::Vector && dHigh < NUM_MIDI_CHANNELS)
+        putData.data.insert = dHigh; // othewise set by file
+    else if (nLow == TOPLEVEL::XML::Instrument)
+    {
+        unsigned char tmp = synth->getRuntime().vectordata.Part;
+        if (dHigh >= NUM_MIDI_PARTS && tmp < NUM_MIDI_PARTS)
+            putData.data.insert = tmp; // last seen by part NRPN
+        else if (dHigh < NUM_MIDI_PARTS)
+            putData.data.insert = dHigh;
+        else
+            return true; // not an error but undefined
+    }
+    synth->interchange.fromMIDI->write(putData.bytes);
+    return true;
+}
+
+
 void MidiDecode::nrpnDirectPart(int dHigh, int par)
 {
     CommandBlock putData;
@@ -538,14 +579,14 @@ void MidiDecode::nrpnDirectPart(int dHigh, int par)
             break;
 
         case 4: // Set part's channel number
-            putData.data.value = par;
+            putData.data.value.F = par;
             putData.data.control = 5;
             putData.data.part = synth->getRuntime().vectordata.Part;
             break;
 
         case 5: // Set part's audio destination
             if (par > 0 and par < 4)
-            putData.data.value = par;
+            putData.data.value.F = par;
             putData.data.control = 120;
             putData.data.part = synth->getRuntime().vectordata.Part;
             putData.data.parameter = 192;
@@ -557,7 +598,7 @@ void MidiDecode::nrpnDirectPart(int dHigh, int par)
                 par = MIN_KEY_SHIFT;
             else if (par > MAX_KEY_SHIFT)
                 par = MAX_KEY_SHIFT;
-            putData.data.value = par;
+            putData.data.value.F = par;
             putData.data.control = 35;
             putData.data.part = synth->getRuntime().vectordata.Part;
             break;
@@ -567,10 +608,10 @@ void MidiDecode::nrpnDirectPart(int dHigh, int par)
     }
     if (dHigh < 4)
         return;
-    //cout << "part " << int(putData.data.part) << "  Chan " << int(par) << endl;
-    putData.data.type = 0xd0;
-
-    synth->midilearn.writeMidi(&putData, sizeof(putData), false);
+    //std::cout << "part " << int(putData.data.part) << "  Chan " << int(par) << std::endl;
+    putData.data.type = 0xc0;
+    putData.data.source = TOPLEVEL::action::toAll;
+    synth->midilearn.writeMidi(&putData, false);
 }
 
 
@@ -620,14 +661,26 @@ void MidiDecode::setMidiBankOrRootDir(unsigned int bank_or_root_num, bool in_pla
             return; // nothing to do!
     }
     else
+    {
         if (bank_or_root_num == synth->getRuntime().currentBank)
             return; // still nothing to do!
+    }
+
+    if (in_place)
+    {
+        if (!setRootDir)
+            synth->setRootBank(UNUSED, bank_or_root_num, false);
+        else
+            synth->setRootBank(bank_or_root_num, UNUSED, false);
+        return;
+    }
 
     CommandBlock putData;
     memset(&putData, 0xff, sizeof(putData));
-    putData.data.value = 0xff;
-    putData.data.type = 0xd0;
-    putData.data.control = 8;
+    putData.data.value.F = 0xff;
+    putData.data.type = 0xc0;
+    putData.data.source = TOPLEVEL::action::toAll;
+    putData.data.control = MIDI::control::bankChange;
     putData.data.part = TOPLEVEL::section::midiIn;
     putData.data.kit = 0;
     putData.data.parameter = 0xc0;
@@ -637,10 +690,7 @@ void MidiDecode::setMidiBankOrRootDir(unsigned int bank_or_root_num, bool in_pla
     else
         putData.data.engine = bank_or_root_num;
 
-    if (in_place)
-        synth->SetRBP(&putData, false);
-    else
-        synth->midilearn.writeMidi(&putData, sizeof(putData), false);
+    synth->midilearn.writeMidi(&putData, false);
 }
 
 
@@ -654,11 +704,12 @@ void MidiDecode::setMidiProgram(unsigned char ch, int prg, bool in_place)
 
     CommandBlock putData;
     memset(&putData, 0xff, sizeof(putData));
-    putData.data.value = prg;
-    putData.data.type = 0xd0;
-    putData.data.control = 8;
+    putData.data.value.F = prg;
+    putData.data.type = 0xc0;
+    putData.data.source = TOPLEVEL::action::toAll;
+    putData.data.control = MIDI::control::instrument;
     putData.data.part = TOPLEVEL::section::midiIn;
-    putData.data.parameter = 0xc0;
+    //putData.data.parameter = 0xc0;
 
     /*
      * This is a bit slow as we send each part individually
@@ -678,11 +729,11 @@ void MidiDecode::setMidiProgram(unsigned char ch, int prg, bool in_place)
                 if (in_place)
                 {
                     synth->partonoffLock(npart, -1);
-                    synth->SetRBP(&putData, false);
+                    synth->setProgramFromBank(&putData, false);
                 }
                 else
                 {
-                    synth->midilearn.writeMidi(&putData, sizeof(putData), false);
+                    synth->midilearn.writeMidi(&putData, false);
                 }
             }
         }
@@ -694,10 +745,10 @@ void MidiDecode::setMidiProgram(unsigned char ch, int prg, bool in_place)
         if (in_place)
         {
             synth->partonoffLock(ch, -1);
-            synth->SetRBP(&putData, false);
+            synth->setProgramFromBank(&putData, false);
         }
         else
-            synth->midilearn.writeMidi(&putData, sizeof(putData), false);
+            synth->midilearn.writeMidi(&putData, false);
     }
 }
 
