@@ -1,7 +1,7 @@
 /*
     MidiLearn.cpp
 
-    Copyright 2016-2018 Will Godfrey
+    Copyright 2016-2020 Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -17,27 +17,44 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    Modified October 2018
 */
 
-#include <iostream>
-#include <cfloat>
-#include <bitset>
-#include <unistd.h>
-#include <list>
-#include <string>
-#include <unistd.h>
-
-using namespace std;
-
 #include "Interface/MidiLearn.h"
 #include "Interface/InterChange.h"
-#include "Misc/MiscFuncs.h"
+#include "Interface/RingBuffer.h"
+#include "Misc/FileMgrFuncs.h"
+#include "Misc/FormatFuncs.h"
+#include "Misc/TextMsgBuffer.h"
 #include "Misc/XMLwrapper.h"
 #include "Misc/SynthEngine.h"
 
+#include <list>
+#include <vector>
+#include <string>
+#include <unistd.h>  // for usleep()
+//#include <iostream>
+
+using file::isRegularFile;
+using file::make_legit_filename;
+using file::make_legit_pathname;
+using file::setExtension;
+
+using func::asString;
+using func::asHexString;
+using std::to_string;
+using std::string;
+using std::vector;
+using std::list;
+
 enum scan : int { noList = -3, listEnd, listBlocked};
 
+
+namespace { // Implementation details...
+
+    TextMsgBuffer& textMsgBuffer = TextMsgBuffer::instance();
+}
+
+
 MidiLearn::MidiLearn(SynthEngine *_synth) :
     learning(false),
     synth(_synth)
@@ -52,26 +69,29 @@ MidiLearn::~MidiLearn()
 }
 
 
-void MidiLearn::setTransferBlock(CommandBlock *getData, string name)
+void MidiLearn::setTransferBlock(CommandBlock *getData)
 {
     //cout << "MIDI Control " << (int) getData->data.control << " Part " << (int) getData->data.part << "  Kit " << (int) getData->data.kit << " Engine " << (int) getData->data.engine << "  Insert " << (int) getData->data.insert << endl;
 
     memcpy(learnTransferBlock.bytes, getData->bytes, sizeof(learnTransferBlock));
-    learnedName = name;
+    learnedName = resolveAll(synth, getData, false);
     learning = true;
-    synth->getRuntime().Log("Learning");
+    synth->getRuntime().Log("Learning " + learnedName);
     updateGui(MIDILEARN::control::sendLearnMessage);
 }
 
 
-bool MidiLearn::runMidiLearn(int _value, unsigned int CC, unsigned char chan, unsigned char category)
+bool MidiLearn::runMidiLearn(int _value, unsigned short int CC, unsigned char chan, bool in_place)
 {
     if (learning)
     {
-        insert(CC, chan);
+        insertLine(CC, chan);
         return true; // block while learning
     }
 
+    if (findSize() == 0)
+        return false; // don't bother if there's no list!
+
     int lastpos = scan::listBlocked;
     LearnBlock foundEntry;
     bool firstLine = true;
@@ -89,7 +109,7 @@ bool MidiLearn::runMidiLearn(int _value, unsigned int CC, unsigned char chan, un
  * equivalent of 0 to 127 under all conditions
  */
         float value; // needs to be refetched each loop
-        if (category & 2)
+        if (CC >= MIDI::CC::identNRPN || CC == MIDI::CC::pitchWheelInner)
         {
             if (status & 16) // 7 bit NRPN
                 value = float(_value & 0x7f);
@@ -104,7 +124,7 @@ bool MidiLearn::runMidiLearn(int _value, unsigned int CC, unsigned char chan, un
         if (minIn > maxIn)
         {
             value = 127 - value;
-            swap(minIn, maxIn);
+            std::swap(minIn, maxIn);
         }
 
         if (minIn == maxIn)
@@ -132,31 +152,31 @@ bool MidiLearn::runMidiLearn(int _value, unsigned int CC, unsigned char chan, un
             value += minOut;
 
         CommandBlock putData;
-        putData.data.value = value;
-        putData.data.type = 0x48 | (foundEntry.data.type & 0x80);
+        putData.data.value.F = value;
+        putData.data.type = 0x40 | (foundEntry.data.type & 0x80);
         // write command from midi with original integer / float type
+        putData.data.source = TOPLEVEL::action::toAll;
         putData.data.control = foundEntry.data.control;
         putData.data.part = foundEntry.data.part;
         putData.data.kit = foundEntry.data.kit;
         putData.data.engine = foundEntry.data.engine;
         putData.data.insert = foundEntry.data.insert;
         putData.data.parameter = foundEntry.data.parameter;
-        putData.data.par2 = foundEntry.data.par2;
-        unsigned int putSize = sizeof(putData);
-        if (writeMidi(&putData, putSize, category & 1))
+        putData.data.miscmsg = foundEntry.data.miscmsg;
+        if (writeMidi(&putData, in_place))
         {
-            if (firstLine && !(category & 1)) // not in_place
+            if (firstLine && !in_place) // not in_place
             // we only want to send an activity once
             // and it's not relevant to jack freewheeling
             {
-                if (CC > 0xff)
+                if (CC >= MIDI::CC::identNRPN)
                     putData.data.type |= 0x10; // mark as NRPN
                 firstLine = false;
                 putData.data.control = MIDILEARN::control::reportActivity;
                 putData.data.part = TOPLEVEL::section::midiLearn;
                 putData.data.kit = (CC & 0xff);
                 putData.data.engine = chan;
-                writeMidi(&putData, putSize, category & 1);
+                writeMidi(&putData, in_place);
             }
         }
         if (lastpos == scan::listBlocked) // blocking all of this CC/chan pair
@@ -166,12 +186,9 @@ bool MidiLearn::runMidiLearn(int _value, unsigned int CC, unsigned char chan, un
 }
 
 
-bool MidiLearn::writeMidi(CommandBlock *putData, unsigned int writesize, bool in_place)
+bool MidiLearn::writeMidi(CommandBlock *putData, bool in_place)
 {
-    char *point = (char*) putData;
-    unsigned int towrite = writesize;
-    unsigned int wrote = 0;
-    unsigned int found;
+    putData->data.source |= TOPLEVEL::action::fromMIDI;
     unsigned int tries = 0;
     bool ok = true;
     if (in_place)
@@ -181,23 +198,16 @@ bool MidiLearn::writeMidi(CommandBlock *putData, unsigned int writesize, bool in
     }
     else
     {
-        if (jack_ringbuffer_write_space(synth->interchange.fromMIDI) >= writesize)
+        do
         {
-            while (towrite && tries < 3)
-            {
-                found = jack_ringbuffer_write(synth->interchange.fromMIDI, point, towrite);
-                wrote += found;
-                point += found;
-                towrite -= found;
-                ++tries;
-            }
-            if (towrite)
-            {
-                ok = false;
-                    synth->getRuntime().Log("Unable to write data to fromMidi buffer", 2);
-            }
+            ++ tries;
+            ok = synth->interchange.fromMIDI->write(putData->bytes);
+            if (!ok)
+                usleep(1);
+        // we can afford a short delay for buffer to clear
         }
-        else
+        while (!ok && tries < 3);
+        if (!ok)
         {
             synth->getRuntime().Log("Midi buffer full!");
             ok = false;
@@ -211,7 +221,7 @@ bool MidiLearn::writeMidi(CommandBlock *putData, unsigned int writesize, bool in
  * This will only be called by incoming midi. It is the only function that
  * needs to be really quick
  */
-int MidiLearn::findEntry(list<LearnBlock> &midi_list, int lastpos, unsigned int CC, unsigned char chan, LearnBlock *block, bool show)
+int MidiLearn::findEntry(list<LearnBlock> &midi_list, int lastpos, unsigned short int CC, unsigned char chan, LearnBlock *block, bool show)
 {
     int newpos = 0; // 'last' comes in at listBlocked for the first call
     list<LearnBlock>::iterator it = midi_list.begin();
@@ -237,7 +247,7 @@ int MidiLearn::findEntry(list<LearnBlock> &midi_list, int lastpos, unsigned int
         if ((it->chan >= 16 || chan == it->chan) && CC == it->CC)
         {
             if (show)
-                synth->getRuntime().Log("Found line " + it->name + "  at " + to_string(newpos)); // a test!
+                synth->getRuntime().Log("Found line " + findName(it) + "  at " + to_string(newpos)); // a test!
             block->chan = it->chan;
             block->CC = it->CC;
             block->min_in = it->min_in;
@@ -307,12 +317,18 @@ void MidiLearn::listLine(int lineNo)
             chan += "All";
         else
             chan += to_string(int(it->chan + 1));
+        string  CCtype;
+        int CC = it->CC;
+        if (CC < 0xff)
+            CCtype = to_string(CC);
+        else
+            CCtype = asHexString((CC >> 7) & 0x7f) + asHexString(CC & 0x7f) + " h";
         synth->getRuntime().Log("Line " + to_string(lineNo + 1) + mute
-                + "  CC " + to_string(int(it->CC))
+                + "  CC " + CCtype
                 + chan
                 + "  Min " + asString(float(it->min_in / 2.0f)) + "%"
                 + "  Max " + asString(float(it->max_in / 2.0f)) + "%"
-                + limit + block + nrpn + "  " + it->name);
+                + limit + block + nrpn + "  " + findName(it));
     }
 }
 
@@ -335,14 +351,14 @@ void MidiLearn::listAll(list<string>& msg_buf)
         if (CC < 0xff)
             CCtype = to_string(CC);
         else
-            CCtype = asHexString((CC >> 8) & 0x7f) + asHexString(CC & 0x7f) + " h";
+            CCtype = asHexString((CC >> 7) & 0x7f) + asHexString(CC & 0x7f) + " h";
         string chan = "  Chan ";
         if ((it->chan) >= NUM_MIDI_CHANNELS)
             chan += "All";
         else
             chan += to_string(int(it->chan + 1));
 
-        msg_buf.push_back("Line " + to_string(lineNo + 1) + "  CC " + CCtype + chan + "  " + it->name);
+        msg_buf.push_back("Line " + to_string(lineNo + 1) + "  CC " + CCtype + chan + "  " + findName(it));
         ++ it;
         ++ lineNo;
     }
@@ -367,10 +383,18 @@ bool MidiLearn::remove(int itemNumber)
 }
 
 
-void MidiLearn::generalOpps(int value, unsigned char type, unsigned char control, unsigned char part, unsigned char _kit, unsigned char engine, unsigned char insert, unsigned char parameter, unsigned char par2)
+void MidiLearn::generalOperations(CommandBlock *getData)
 {
-    unsigned int kit = part; // to silence warning
-    kit = _kit; // may need to set as an NRPN
+    int value = getData->data.value.F;
+    unsigned char type = getData->data.type;
+    unsigned char control = getData->data.control;
+//    unsigned char part = getData->data.part;
+    unsigned int kit = getData->data.kit; // may need to set as an NRPN
+    unsigned char engine = getData->data.engine;
+    unsigned char insert = getData->data.insert;
+    unsigned char parameter = getData->data.parameter;
+    unsigned char par2 = getData->data.miscmsg;
+
     if (control == MIDILEARN::control::sendRefreshRequest)
     {
         updateGui();
@@ -381,7 +405,7 @@ void MidiLearn::generalOpps(int value, unsigned char type, unsigned char control
     if (control == MIDILEARN::control::clearAll)
     {
         midi_list.clear();
-        synth->setLastfileAdded(XML_MIDILEARN, "");
+        synth->setLastfileAdded(TOPLEVEL::XML::MLearn, "");
         updateGui();
         synth->getRuntime().Log("List cleared");
         return;
@@ -390,7 +414,7 @@ void MidiLearn::generalOpps(int value, unsigned char type, unsigned char control
     string name;
     if (control == MIDILEARN::control::loadList)
     {
-        name = (miscMsgPop(par2));
+        name = (textMsgBuffer.fetch(par2));
         if (loadList(name))
             synth->getRuntime().Log("Loaded " + name);
         updateGui();
@@ -400,7 +424,7 @@ void MidiLearn::generalOpps(int value, unsigned char type, unsigned char control
     if (control == MIDILEARN::control::loadFromRecent)
     {
         int pos = 0;
-        vector<string> &listtype = *synth->getHistory(XML_MIDILEARN);
+        vector<string> &listtype = *synth->getHistory(TOPLEVEL::XML::MLearn);
         vector<string>::iterator it = listtype.begin();
         while (it != listtype.end() && pos != value)
         {
@@ -409,7 +433,7 @@ void MidiLearn::generalOpps(int value, unsigned char type, unsigned char control
         }
         if (it == listtype.end())
         {
-            synth->getRuntime().Log("No entry for number " + to_string(int(par2 + 1)));
+            synth->getRuntime().Log("No entry for number " + to_string(int(value + 1)));
         }
         else
         {
@@ -423,7 +447,7 @@ void MidiLearn::generalOpps(int value, unsigned char type, unsigned char control
     }
     if (control == MIDILEARN::control::saveList)
     {
-        name = (miscMsgPop(par2));
+        name = (textMsgBuffer.fetch(par2));
         if (saveList(name))
             synth->getRuntime().Log("Saved " + name);
         synth->getRuntime().finishedCLI = true;
@@ -434,7 +458,7 @@ void MidiLearn::generalOpps(int value, unsigned char type, unsigned char control
         learning = false;
         synth->getRuntime().finishedCLI = true;
         synth->getRuntime().Log("Midi Learn cancelled");
-        updateGui(control);
+        updateGui(MIDILEARN::control::cancelLearn);
         return;
     }
 
@@ -457,22 +481,22 @@ void MidiLearn::generalOpps(int value, unsigned char type, unsigned char control
         return;
     }
 
-    if (insert == 0xff) // don't change
+    if (insert == UNUSED) // don't change
         insert = it->min_in;
     else
         lineName = "Min = " + asString(float(insert / 2.0f)) + "%";
 
-    if (parameter == 0xff)
+    if (parameter == UNUSED)
         parameter = it->max_in;
     else
         lineName = "Max = " + asString(float(parameter / 2.0f)) + "%";
 
-    if (kit == 255 || it->CC > 0xff) // might be an NRPN
+    if (kit == UNUSED || it->CC > 0xff) // might be an NRPN
         kit = it->CC; // remember NRPN has a high bit set
     else
         lineName = "CC = " + to_string(int(kit));
 
-    if (engine == 255)
+    if (engine == UNUSED)
         engine = it->chan;
     else
     {
@@ -482,7 +506,6 @@ void MidiLearn::generalOpps(int value, unsigned char type, unsigned char control
             lineName = "Chan = " + to_string(int(engine) + 1);
     }
 
-
     if (control == MIDILEARN::control::CCorChannel)
     {
         bool moveLine = true;
@@ -571,10 +594,11 @@ void MidiLearn::generalOpps(int value, unsigned char type, unsigned char control
                 name += " disabled";
             synth->getRuntime().Log("Line " + to_string(lineNo + 1) + " " + name);
         }
+
         CommandBlock putData;
         memset(&putData.bytes, 255, sizeof(putData));
         // need to work on this more
-        putData.data.value = value;
+        putData.data.value.F = value;
         putData.data.type = type;
         putData.data.control = MIDILEARN::control::ignoreMove;
         putData.data.kit = kit;
@@ -599,7 +623,6 @@ void MidiLearn::generalOpps(int value, unsigned char type, unsigned char control
         entry.status = type;
         entry.min_out = it->min_out;
         entry.max_out = it->max_out;
-        entry.name = it->name;
         entry.data = it->data;
         unsigned int CC = entry.CC;
         int chan = entry.chan;
@@ -635,7 +658,25 @@ void MidiLearn::generalOpps(int value, unsigned char type, unsigned char control
 }
 
 
-void MidiLearn::insert(unsigned int CC, unsigned char chan)
+string MidiLearn::findName(list<LearnBlock>::iterator it)
+{
+    CommandBlock putData;
+    putData.data.value.I = 0;
+    putData.data.source = 0;
+
+    putData.data.type = it->data.type;
+    putData.data.control = it->data.control;
+    putData.data.part = it->data.part;
+    putData.data.kit = it->data.kit;
+    putData.data.engine = it->data.engine;
+    putData.data.insert = it->data.insert;
+    putData.data.parameter = it->data.parameter;
+    putData.data.offset = UNUSED;
+    string name = resolveAll(synth, &putData, false);;
+    return name;
+}
+
+void MidiLearn::insertLine(unsigned short int CC, unsigned char chan)
 {
     /*
      * This will eventually be part of a paging system of
@@ -646,27 +687,27 @@ void MidiLearn::insert(unsigned int CC, unsigned char chan)
         CommandBlock putData;
         int putSize = sizeof(putData);
         memset(&putData, 0xff, putSize);
-        putData.data.value = 0;
+        putData.data.value.F = 0;
+        putData.data.source = TOPLEVEL::action::toAll;
         putData.data.type = 0xc8;
-        putData.data.control = 0xfe; // TODO don't understand this :(
+        putData.data.control = TOPLEVEL::control::textMessage;
         putData.data.part = TOPLEVEL::section::midiIn;
         putData.data.parameter = 0x80;
-        putData.data.par2 = miscMsgPush("Midi Learn full!");
-        writeMidi(&putData, putSize, false);
+        putData.data.miscmsg = textMsgBuffer.push("Midi Learn full!");
+        writeMidi(&putData, false);
         learning = false;
         return;
     }
 
-    unsigned char stat = 0;
-    if (CC > 0xff)
-        stat |= 9; // mark as NRPN and set 'block'
+    unsigned char status = 0;
+    if (CC >= MIDI::CC::identNRPN)
+        status |= 9; // mark as NRPN and set 'block'
     LearnBlock entry;
     entry.chan = chan;
     entry.CC = CC;
     entry.min_in = 0;
     entry.max_in = 200;
-    entry.status = stat;
-    entry.name = learnedName;
+    entry.status = status;
 
     //cout << "SEND Control " << (int) learnTransferBlock.data.control << " Part " << (int) learnTransferBlock.data.part << "  Kit " << (int) learnTransferBlock.data.kit << " Engine " << (int) learnTransferBlock.data.engine << "  Insert " << (int) learnTransferBlock.data.insert << endl;
 
@@ -685,7 +726,7 @@ void MidiLearn::insert(unsigned int CC, unsigned char chan)
     entry.data.engine = learnTransferBlock.data.engine;
     entry.data.insert = learnTransferBlock.data.insert;
     entry.data.parameter = learnTransferBlock.data.parameter;
-    entry.data.par2 = learnTransferBlock.data.par2;
+    entry.data.miscmsg = learnTransferBlock.data.miscmsg;
 
     list<LearnBlock>::iterator it;
     it = midi_list.begin();
@@ -708,14 +749,14 @@ void MidiLearn::insert(unsigned int CC, unsigned char chan)
     else
         midi_list.insert(it, entry);
 
-    synth->getRuntime().Log("Learned ");
+    //synth->getRuntime().Log("Learned ");
     unsigned int CCh = entry.CC;
     string CCtype;
     if (CCh < 0xff)
         CCtype = "CC " + to_string(CCh);
     else
         CCtype = "NRPN " + asHexString((CCh >> 7) & 0x7f) + " " + asHexString(CCh & 0x7f);
-    synth->getRuntime().Log(CCtype + "  Chan " + to_string((int)entry.chan + 1) + "  " + entry.name);
+    synth->getRuntime().Log("Learned " + CCtype + "  Chan " + to_string((int)entry.chan + 1) + "  " + learnedName);
     updateGui(MIDILEARN::control::limit);
     learning = false;
 }
@@ -726,26 +767,18 @@ void MidiLearn::writeToGui(CommandBlock *putData)
     if (!synth->getRuntime().showGui)
         return;
     putData->data.part = TOPLEVEL::section::midiLearn;
-    unsigned int writesize = sizeof(*putData);
-    char *point = (char*)putData;
-    unsigned int towrite = writesize;
-    unsigned int wrote = 0;
-    unsigned int found = 0;
-    unsigned int tries = 0;
-    if (jack_ringbuffer_write_space(synth->interchange.toGUI) >= writesize)
+    int tries = 0;
+    bool ok = false;
+    do
     {
-        while (towrite && tries < 3)
-        {
-            found = jack_ringbuffer_write(synth->interchange.toGUI, point, towrite);
-            wrote += found;
-            point += found;
-            towrite -= found;
-            ++tries;
-        }
-        if (towrite)
-            synth->getRuntime().Log("Unable to write data to toGui buffer", 2);
+        ok = synth->interchange.toGUI->write(putData->bytes);
+        ++tries;
+        if (!ok)
+                usleep(1);
+        // we can afford a short delay for buffer to clear
     }
-    else
+    while (!ok && tries < 3);
+    if(!ok)
         synth->getRuntime().Log("toGui buffer full!", 2);
 }
 
@@ -755,27 +788,27 @@ void MidiLearn::updateGui(int opp)
     if (!synth->getRuntime().showGui)
         return;
     CommandBlock putData;
-    if (opp == 21)
+    if (opp == MIDILEARN::control::sendLearnMessage)
     {
         putData.data.control = MIDILEARN::control::sendLearnMessage;
-        putData.data.par2 = miscMsgPush("Learning " + learnedName);
+        putData.data.miscmsg = textMsgBuffer.push("Learning " + learnedName);
     }
-    else if (opp == 255)
+    else if (opp == MIDILEARN::control::cancelLearn)
     {
         putData.data.control = MIDILEARN::control::cancelLearn;
-        putData.data.par2 = 0xff;
+        putData.data.miscmsg = NO_MSG;
     }
     else
     {
         putData.data.control = MIDILEARN::control::clearAll;
-        putData.data.par2 = 0xff;
-        if (opp == 2)
-            putData.data.kit = 2; // close editing window
+        putData.data.miscmsg = NO_MSG;
+        if (opp == MIDILEARN::control::hideGUI)
+            return;
     }
-    putData.data.value = 0;
+    putData.data.value.F = 0;
     writeToGui(&putData);
 
-    if (opp > 2) // sending back message gui
+    if (opp >= MIDILEARN::control::hideGUI) // just sending back gui message
         return;
 
     int lineNo = 0;
@@ -783,26 +816,27 @@ void MidiLearn::updateGui(int opp)
     it = midi_list.begin();
     while (it != midi_list.end())
     {
-        unsigned int newCC = it->CC;
-        putData.data.value = lineNo;
+        unsigned short int newCC = (it->CC) & MIDI::CC::maxNRPN;
+        putData.data.value.F = lineNo;
         putData.data.type = it->status;
+        putData.data.source = TOPLEVEL::action::toAll;
         putData.data.control = MIDILEARN::control::CCorChannel;
         putData.data.kit = (newCC & 0xff);
         putData.data.engine = it->chan;
         putData.data.insert = it->min_in;
         putData.data.parameter = it->max_in;
-        putData.data.par2 = miscMsgPush(it->name);
+        putData.data.miscmsg = textMsgBuffer.push(findName(it));
         writeToGui(&putData);
-        if (newCC > 0xff || (it->status & 8) > 0)
-        { // status now used in case NRPN is < 0x100
+        if (it->status & 8)
+        { // status used in case NRPN is < 0x100
             putData.data.control = MIDILEARN::control::nrpnDetected; // it's an NRPN
-            putData.data.engine = ((newCC >> 8) & 0xff);
+            putData.data.engine = (newCC >> 8);
             writeToGui(&putData);
         }
         ++it;
         ++lineNo;
     }
-    if (opp == 1 && synth->getRuntime().showLearnedCC == true) // open the gui editing window
+    if (synth->getRuntime().showLearnedCC == true) // open the gui editing window
     {
         putData.data.control = MIDILEARN::control::sendRefreshRequest;
         writeToGui(&putData);
@@ -824,19 +858,19 @@ bool MidiLearn::saveList(string name)
         return false;
     }
 
-    string file = setExtension(name, "xly");
-    legit_pathname(file);
+    string file = setExtension(name, EXTEN::mlearn);
+    make_legit_pathname(file);
 
-    synth->getRuntime().xmlType = XML_MIDILEARN;
+    synth->getRuntime().xmlType = TOPLEVEL::XML::MLearn;
     XMLwrapper *xml = new XMLwrapper(synth, true);
     if (!xml)
     {
-        synth->getRuntime().Log("Save Midi Learn failed xmltree allocation");
+        synth->getRuntime().Log("Save Midi Learn failed xml allocation");
         return false;
     }
-    bool ok = insertMidiListData(true,  xml);
+    bool ok = insertMidiListData(xml);
     if (xml->saveXMLfile(file))
-        synth->addHistory(file, 6);
+        synth->addHistory(file, TOPLEVEL::XML::MLearn);
     else
     {
         synth->getRuntime().Log("Failed to save data to " + file);
@@ -847,12 +881,11 @@ bool MidiLearn::saveList(string name)
 }
 
 
-bool MidiLearn::insertMidiListData(bool full,  XMLwrapper *xml)
+bool MidiLearn::insertMidiListData(XMLwrapper *xml)
 {
-    int ID = full; // to silence warning
     if (midi_list.size() == 0)
         return false;
-    ID = 0;
+    int ID = 0;
     list<LearnBlock>::iterator it;
     it = midi_list.begin();
     xml->beginbranch("MIDILEARN");
@@ -886,8 +919,8 @@ bool MidiLearn::insertMidiListData(bool full,  XMLwrapper *xml)
                 xml->addpar("Engine", it->data.engine);
                 xml->addpar("Insert", it->data.insert);
                 xml->addpar("Parameter", it->data.parameter);
-                xml->addpar("Secondary_Parameter", it->data.par2);
-                xml->addparstr("Command_Name", it->name.c_str());
+                xml->addpar("Secondary_Parameter", it->data.miscmsg);
+                xml->addparstr("Command_Name",findName(it).c_str());
                 xml->endbranch();
             xml->endbranch();
             ++it;
@@ -905,9 +938,9 @@ bool MidiLearn::loadList(string name)
         synth->getRuntime().Log("No filename");
         return false;
     }
-    string file = setExtension(name, "xly");
-    legit_pathname(file);
-    if (!isRegFile(file))
+    string file = setExtension(name, EXTEN::mlearn);
+    make_legit_pathname(file);
+    if (!isRegularFile(file))
     {
         synth->getRuntime().Log("Can't find " + file);
         return false;
@@ -923,7 +956,7 @@ bool MidiLearn::loadList(string name)
     delete xml;
     if (!ok)
         return false;
-    synth->addHistory(file, 6);
+    synth->addHistory(file, TOPLEVEL::XML::MLearn);
     return true;
 }
 
@@ -937,15 +970,14 @@ bool MidiLearn::extractMidiListData(bool full,  XMLwrapper *xml)
         return false;
     }
     LearnBlock entry;
-    CommandBlock real;
     midi_list.clear();
     int ID = 0;
     int status;
-    unsigned int ent;
+    unsigned int ident;
     while (true)
     {
         status = 0;
-        ent = 0;
+        ident = 0;
         if (!xml->enterbranch("LINE", ID))
             break;
         else
@@ -954,30 +986,20 @@ bool MidiLearn::extractMidiListData(bool full,  XMLwrapper *xml)
                 status |= 4;
             if (xml->getparbool("NRPN", 0))
             {
-                ent = 0x10000; // set top bit for NRPN comparison
+                ident = MIDI::CC::identNRPN; // set top bit for NRPN indication
                 status |= 8;
             }
             if (xml->getparbool("7_bit",0))
                 status |= 16;
 
-            ent |= xml->getpar("Midi_Controller", 0, 0, 0xfffff);
-            if (ent > 0xff) // belt & braces!
-            {
-                ent |= 0x10000;
-                status |= 8;
-            }
-            entry.CC = ent;
+            entry.CC = ident | xml->getpar("Midi_Controller", 0, 0, MIDI::CC::maxNRPN);
 
             entry.chan = xml->getpar127("Midi_Channel", 0);
 
-            int min = int((xml->getparreal("Midi_Min", 200.0f) * 1.575f) + 0.5f);
-            if (min >= 200)
-                min = int((xml->getpar127("Midi_Min", 0) * 1.575f) + 0.5f);
+            int min = int((xml->getparreal("Midi_Min", 200.0f) * 1.575f) + 0.1f);
             entry.min_in = min;
 
-            int max = int((xml->getparreal("Midi_Max", 200.0f) * 1.575f) + 0.5f);
-            if (max >= 200)
-                max = int((xml->getpar127("Midi_Max", 127) * 1.575f)+ 0.5f);
+            int max = int((xml->getparreal("Midi_Max", 200.0f) * 1.575f) + 0.1f);
             entry.max_in = max;
 
             if (xml->getparbool("Limit",0))
@@ -988,24 +1010,16 @@ bool MidiLearn::extractMidiListData(bool full,  XMLwrapper *xml)
             entry.max_out = xml->getpar("Convert_Max", 0, -16384, 16383);
             xml->enterbranch("COMMAND");
                 entry.data.type = xml->getpar255("Type", 0); // ??
-                real.data.control = entry.data.control = xml->getpar255("Control", 0);
-                real.data.part = entry.data.part = xml->getpar255("Part", 0);
-                real.data.kit = entry.data.kit = xml->getpar255("Kit_Item", 0);
-                real.data.engine = entry.data.engine = xml->getpar255("Engine", 0);
-                real.data.insert = entry.data.insert = xml->getpar255("Insert", 0);
-                real.data.parameter = entry.data.parameter = xml->getpar255("Parameter", 0);
-                real.data.par2 = entry.data.par2 = xml->getpar255("Secondary_Parameter", 0);
+                entry.data.control = xml->getpar255("Control", 0);
+                entry.data.part = xml->getpar255("Part", 0);
+                entry.data.kit = xml->getpar255("Kit_Item", 0);
+                entry.data.engine = xml->getpar255("Engine", 0);
+                entry.data.insert = xml->getpar255("Insert", 0);
+                entry.data.parameter = xml->getpar255("Parameter", 0);
+                entry.data.miscmsg = xml->getpar255("Secondary_Parameter", 0);
                 xml->exitbranch();
             xml->exitbranch();
             entry.status = status;
-            real.data.value = FLT_MAX;
-            real.data.type = 0;
-            /*
-             * Need to improve resolveReplies so that we can
-             * get just the name without corrupting data
-             */
-            synth->interchange.resolveReplies(&real);
-            entry.name = miscMsgPop(real.data.par2);
             midi_list.push_back(entry);
             ++ ID;
         }
