@@ -2,7 +2,7 @@
     YoshimiLV2Plugin
 
     Copyright 2014, Andrew Deryabin <andrewderyabin@gmail.com>
-    Copyright 2016-2018, Will Godfrey & others.
+    Copyright 2016-2020, Will Godfrey & others.
 
     This file is part of yoshimi, which is free software: you can
     redistribute it and/or modify it under the terms of the GNU General
@@ -17,7 +17,6 @@
     You should have received a copy of the GNU General Public License
     along with yoshimi.  If not, see <http://www.gnu.org/licenses/>.
 
-    Modified October 2018
 */
 
 #include "YoshimiLV2Plugin.h"
@@ -25,14 +24,12 @@
 #include "Misc/ConfBuild.h"
 #include "Misc/SynthEngine.h"
 #include "Interface/InterChange.h"
+#include "Interface/Data2Text.h"
 #include "Interface/MidiDecode.h"
 #include "MusicIO/MusicClient.h"
-#include "MasterUI.h"
-#include <math.h>
-#include <stdio.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
+#ifdef GUI_FLTK
+    #include "MasterUI.h"
+#endif
 
 
 #define YOSHIMI_STATE_URI "http://yoshimi.sourceforge.net/lv2_plugin#state"
@@ -42,6 +39,7 @@
 
 #define YOSHIMI_LV2_BUF_SIZE__maxBlockLength      YOSHIMI_LV2_BUF_SIZE_PREFIX "maxBlockLength"
 #define YOSHIMI_LV2_BUF_SIZE__minBlockLength      YOSHIMI_LV2_BUF_SIZE_PREFIX "minBlockLength"
+#define YOSHIMI_LV2_BUF_SIZE__nominalBlockLength      YOSHIMI_LV2_BUF_SIZE_PREFIX "nominalBlockLength"
 
 #define YOSHIMI_LV2_OPTIONS_URI    "http://lv2plug.in/ns/ext/options"
 #define YOSHIMI_LV2_OPTIONS_PREFIX YOSHIMI_LV2_OPTIONS_URI "#"
@@ -52,7 +50,7 @@
 #define YOSHIMI_LV2_STATE__StateChanged      "http://lv2plug.in/ns/ext/state#StateChanged"
 
 extern SynthEngine *firstSynth;
-extern int startInstance;
+
 
 typedef enum {
     LV2_OPTIONS_INSTANCE,
@@ -108,20 +106,18 @@ void YoshimiLV2Plugin::process(uint32_t sample_count)
     {
         return;
     }
-    //synth->sent_all_buffersize_f = min(sample_count, (uint32_t)synth->buffersize);
+
     /*
-     * The line above seems to cause problems with envelopes
-     * in Carla.
-     * It has been commented out and investigation is ongoing
-     * to ensure it's removal doesn't cause other problems.
+     * Our implimentation of LV2 has a problem with envelopes. In general
+     * the bigger the buffer size the shorter the envelope, and whichever
+     * is the smallest (host size or Yoshimi size) determines the time.
+     *
+     * However, Yoshimi is always correct when working standalone.
      */
 
-    //int real_sample_count = sample_count;
-    int real_sample_count = min(sample_count, _bufferSize);
-    // not sure which of the above two is the best :(
     int offs = 0;
-    int next_frame = 0;
-    int processed = 0;
+    uint32_t next_frame = 0;
+    uint32_t processed = 0;
     float *tmpLeft [NUM_MIDI_PARTS + 1];
     float *tmpRight [NUM_MIDI_PARTS + 1];
     struct midi_event intMidiEvent;
@@ -144,18 +140,18 @@ void YoshimiLV2Plugin::process(uint32_t sample_count)
         if (event->body.type == _midi_event_id)
         {
             next_frame = event->time.frames;
-            if (next_frame >= real_sample_count)
+            if (next_frame >= sample_count)
                 continue;
             /*if (next_frame == _bufferSize - 1
                && processed == 0)
             {
                 next_frame = 0;
             }*/
-            int to_process = next_frame - offs;
+            uint32_t to_process = next_frame - offs;
 
             if ((to_process > 0)
-               && (processed < real_sample_count)
-               && (to_process <= (real_sample_count - processed)))
+               && (processed < sample_count)
+               && (to_process <= (sample_count - processed)))
             {
                 int mastered = 0;
                 offs = next_frame;
@@ -179,9 +175,9 @@ void YoshimiLV2Plugin::process(uint32_t sample_count)
         }
     }
 
-    if (processed < real_sample_count)
+    if (processed < sample_count)
     {
-        uint32_t to_process = real_sample_count - processed;
+        uint32_t to_process = sample_count - processed;
         int mastered = 0;
         offs = next_frame;
         while (to_process - mastered > 0)
@@ -291,6 +287,7 @@ YoshimiLV2Plugin::YoshimiLV2Plugin(SynthEngine *synth, double sampleRate, const
         ++features;
     }
 
+    uint32_t nomBufSize = 0;
     if (_uridMap.map != NULL && options != NULL)
     {
         _midi_event_id = _uridMap.map(_uridMap.handle, LV2_MIDI__MidiEvent);
@@ -298,6 +295,7 @@ YoshimiLV2Plugin::YoshimiLV2Plugin(SynthEngine *synth, double sampleRate, const
         _atom_string_id = _uridMap.map(_uridMap.handle, LV2_ATOM__String);
         LV2_URID maxBufSz = _uridMap.map(_uridMap.handle, YOSHIMI_LV2_BUF_SIZE__maxBlockLength);
         LV2_URID minBufSz = _uridMap.map(_uridMap.handle, YOSHIMI_LV2_BUF_SIZE__minBlockLength);
+        LV2_URID nomBufSz = _uridMap.map(_uridMap.handle, YOSHIMI_LV2_BUF_SIZE__nominalBlockLength);
         LV2_URID atomInt = _uridMap.map(_uridMap.handle, LV2_ATOM__Int);
         _atom_type_chunk = _uridMap.map(_uridMap.handle, LV2_ATOM__Chunk);
         _atom_type_sequence = _uridMap.map(_uridMap.handle, LV2_ATOM__Sequence);
@@ -314,12 +312,17 @@ YoshimiLV2Plugin::YoshimiLV2Plugin(SynthEngine *synth, double sampleRate, const
                     if (_bufferSize < bufSz)
                         _bufferSize = bufSz;
                 }
+                if (options->key == nomBufSz && options->type == atomInt)
+                    nomBufSize = *static_cast<const uint32_t *>(options->value);
             }
             ++options;
         }
     }
 
-    if (_bufferSize == 0)
+    //_synth->getRuntime().Log("Buffer size " + to_string(nomBufSize));;
+    if (nomBufSize > 0)
+        _bufferSize = nomBufSize;
+    else if (_bufferSize == 0)
         _bufferSize = MAX_BUFFER_SIZE;
 }
 
@@ -365,7 +368,7 @@ bool YoshimiLV2Plugin::init()
 
     _synth->getRuntime().runSynth = true;
 
-    if (!_synth->getRuntime().startThread(&_pIdleThread, YoshimiLV2Plugin::static_idleThread, this, false, 0, false, "LV2 idle"))
+    if (!_synth->getRuntime().startThread(&_pIdleThread, YoshimiLV2Plugin::static_idleThread, this, false, 0, "LV2 idle"))
     {
         synth->getRuntime().Log("Failed to start idle thread");
         return false;
@@ -379,8 +382,18 @@ bool YoshimiLV2Plugin::init()
 LV2_Handle	YoshimiLV2Plugin::instantiate (const struct _LV2_Descriptor *desc, double sample_rate, const char *bundle_path, const LV2_Feature *const *features)
 {
     SynthEngine *synth = new SynthEngine(0, NULL, true);
-    if (synth == NULL)
+    if (synth == NULL || !synth->getRuntime().isRuntimeSetupCompleted()){
         return NULL;
+    }
+    Fl::lock();
+    /*
+     * Perform further global initialisation.
+     * For stand-alone the equivalent init happens in main(),
+     * after mainCreateNewInstance() returned successfully.
+     */
+    synth->installBanks();
+    synth->loadHistory();
+
     YoshimiLV2Plugin *inst = new YoshimiLV2Plugin(synth, sample_rate, bundle_path, features, desc);
     if (inst->init())
         return static_cast<LV2_Handle>(inst);
@@ -768,12 +781,7 @@ void YoshimiLV2PluginUI::show()
         return;
     }
     if (bInit)
-    {
-        Fl::lock();
-        Fl::unlock();
         _masterUI->Init(uiHost.plugin_human_id);
-    }
-
 }
 
 
@@ -844,7 +852,7 @@ extern "C" const LV2UI_Descriptor* lv2ui_descriptor(uint32_t index)
 }
 
 
-int mainCreateNewInstance(unsigned int, bool) //stub
+int mainCreateNewInstance(unsigned int) //stub
 {
     return 0;
 }
