@@ -4,7 +4,7 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2017-2018, Will Godfrey
+    Copyright 2017-2019, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
     This file is derivative of original ZynAddSubFX code.
 
-    Modified October 2018
+    Modified May 2019
 */
 
 #include <cmath>
@@ -34,8 +34,56 @@
 #include "Misc/XMLwrapper.h"
 #include "Misc/Microtonal.h"
 #include "Misc/SynthEngine.h"
+#include "Misc/FormatFuncs.h"
+#include "Misc/FileMgrFuncs.h"
+
+using file::loadText;
+using file::findLeafName;
+using std::cout;
+using std::endl;
+
+
+namespace { // local implementation details
+
+    const size_t BUFFSIZ = 500;      // for loading KBM or SCL settings from a file
+    const size_t MAX_LINE_SIZE = 80; // for converting text to mappings or tunings
+
+
+    inline std::string lineInText(std::string text, size_t &point)
+    {
+        size_t len = text.length();
+        if (point >= len - 1)
+            return "";
+        size_t it = 0;
+        while (it < len - point && text.at(point + it) >= ' ')
+            ++it;
+        std::string line = text.substr(point, it);
+        point += (it + 1);
+        return line;
+    }
+
+
+    inline bool C_lineInText(std::string text, size_t &point, char *line, size_t length)
+    {
+        bool ok = true;
+        std::string found = lineInText(text, point);
+        if (found == "")
+            line[0] = 0;
+        else if (found.length() < (length - 1))
+        {
+            strcpy(line, found.c_str());
+            line[length] = 0;
+        }
+        else
+        {
+            ok = false;
+            line[0] = 0;
+        }
+        return ok;
+    }
+
+}//(End)implementation details
 
-#define MAX_LINE_SIZE 80
 
 void Microtonal::defaults(void)
 {
@@ -43,8 +91,8 @@ void Microtonal::defaults(void)
     Pinvertupdowncenter = 60;
     octavesize = 12;
     Penabled = 0;
-    PAnote = 69;
-    PAfreq = 440.0f;
+    PrefNote = 69;
+    PrefFreq = 440.0f;
     Pscaleshift = 64;
 
     Pfirstkey = 0;
@@ -56,9 +104,9 @@ void Microtonal::defaults(void)
     for (int i = 0; i < 128; ++i)
         Pmapping[i] = i;
 
-    for (int i = 0; i < MAX_OCTAVE_SIZE; ++i)
+    for (size_t i = 0; i < MAX_OCTAVE_SIZE; ++i)
     {
-        octave[i].text = reformatline(to_string((i % octavesize + 1) * 100)+ ".0");
+        octave[i].text = reformatline(std::to_string((i % octavesize + 1) * 100)+ ".0");
         octave[i].tuning = tmpoctave[i].tuning = pow(2.0, (i % octavesize + 1) / 12.0);
         octave[i].type = tmpoctave[i].type = 1;
         octave[i].x1 = tmpoctave[i].x1 = (i % octavesize + 1) * 100;
@@ -115,7 +163,7 @@ float Microtonal::getNoteFreq(int note, int keyshift)
             return -1.0f;
         // Compute how many mapped keys are from middle note to reference note
         // and find out the proportion between the freq. of middle note and "A" note
-        int tmp = PAnote - Pmiddlenote, minus = 0;
+        int tmp = PrefNote - Pmiddlenote, minus = 0;
         if (tmp < 0)
         {
             tmp   = -tmp;
@@ -155,17 +203,18 @@ float Microtonal::getNoteFreq(int note, int keyshift)
 
         freq = (degkey == 0) ? (1.0f) : octave[degkey - 1].tuning;
         freq *= powf(octave[octavesize - 1].tuning, degoct);
-        freq *= PAfreq / rap_anote_middlenote;
+        freq *= PrefFreq / rap_anote_middlenote;
     }
     else // if the mapping is disabled
     {
-        int nt = note - PAnote + scaleshift;
+        int nt = note - PrefNote + scaleshift;
         int ntkey = (nt + octavesize * 100) % octavesize;
-        int ntoct = (nt - ntkey) / octavesize;
+        // cast octavesize to a signed type so the expression stays signed
+        int ntoct = (nt - ntkey) / int(octavesize);
 
         float oct  = octave[octavesize - 1].tuning;
         freq = octave[(ntkey + octavesize - 1) % octavesize].tuning
-               * powf(oct, ntoct) * PAfreq;
+               * powf(oct, ntoct) * PrefFreq;
         if (ntkey == 0)
             freq /= oct;
     }
@@ -258,8 +307,7 @@ int Microtonal::linetotunings(unsigned int nline, const char *line)
         case 1:
             x1 = (int) floor(x);
             tmp = fmod(x, 1.0);
-            FR2Z2I(floor(tmp * 1e6), x2);
-            //x2 = (int)truncf(floor(tmp * 1e6));
+            x2 = int(floor(tmp * 1e6));
             tuning = pow(2.0, x / 1200.0);
             break;
         case 2:
@@ -281,7 +329,7 @@ int Microtonal::linetotunings(unsigned int nline, const char *line)
 // Convert the text to tunings
 int Microtonal::texttotunings(const char *text)
 {
-    int i;
+    size_t i;
     unsigned int k = 0, nl = 0;
     char *lin;
 
@@ -359,7 +407,7 @@ int Microtonal::texttomapping(const char *text)
     if (tx)
     {
         Pmapsize = tx;
-        swap(Pmapping, tmpMap);
+        std::swap(Pmapping, tmpMap);
     }
     else
         return -6;
@@ -377,13 +425,13 @@ string Microtonal::keymaptotext(void)
         if (Pmapping[i] == -1)
             text += "x";
         else
-            text += to_string(Pmapping[i]);
+            text += std::to_string(Pmapping[i]);
     }
     return text;
 }
 
 // Convert tuning to text line
-void Microtonal::tuningtoline(int n, char *line, int maxn)
+void Microtonal::tuningtoline(unsigned int n, char *line, int maxn)
 {
     if (n > octavesize || n > MAX_OCTAVE_SIZE)
     {
@@ -407,7 +455,7 @@ string Microtonal::tuningtotext()
 {
     string text;
     char *buff = new char[100];
-    for (int i = 0; i < octavesize; ++i)
+    for (size_t i = 0; i < octavesize; ++i)
     {
         if (i > 0)
             text += "\n";
@@ -419,10 +467,12 @@ string Microtonal::tuningtotext()
 }
 
 
-int Microtonal::loadline(FILE *file, char *line)
+int Microtonal::loadLine(string text, size_t &point, char *line, size_t maxlen)
 {
     do {
-        if (!fgets(line, 500, file))
+        line[0] = 0;
+        C_lineInText(text, point, line, maxlen);
+        if (line[0] == 0)
             return -5;
     } while (line[0] == '!');
     return 0;
@@ -432,33 +482,33 @@ int Microtonal::loadline(FILE *file, char *line)
 // Loads the tunings from a scl file
 int Microtonal::loadscl(string filename)
 {
-    FILE *file = fopen(filename.c_str(), "r");
-    if (!file)
-        return -3; // last time we can return before fclose
-    char tmp[500];
+    string text = loadText(filename);
+    if (text == "")
+        return -3;
+    char tmp[BUFFSIZ];
+    size_t point = 0;
     int err = 0;
     int nnotes;
 
-    fseek(file, 0, SEEK_SET);
     // loads the short description
-    if (loadline(file, &tmp[0]))
+    if (loadLine(text, point, tmp, BUFFSIZ))
         err = -4;
     if (err == 0)
     {
         for (int i = 0; i < 500; ++i)
             if (tmp[i] < 32)
                 tmp[i] = 0;
-        Pname = findleafname(filename);
+        Pname = findLeafName(filename);
         Pcomment = string(tmp);
         // loads the number of the notes
-        if (loadline(file, &tmp[0]))
+        if (loadLine(text, point, tmp, BUFFSIZ))
             err = -5;
     }
     if (err == 0)
     {
         nnotes = MAX_OCTAVE_SIZE;
         sscanf(&tmp[0], "%d", &nnotes);
-        if (nnotes > MAX_OCTAVE_SIZE || nnotes < 2)
+        if (size_t(nnotes) > MAX_OCTAVE_SIZE || nnotes < 2)
             err = -6;
     }
     if (err == 0)
@@ -466,19 +516,18 @@ int Microtonal::loadscl(string filename)
     // load the tunings
         for (int nline = 0; nline < nnotes; ++nline)
         {
-            err = loadline(file, &tmp[0]);
+            err = loadLine(text, point, tmp, BUFFSIZ);
             if (err == 0)
                 err = linetotunings(nline, &tmp[0]);
             if (err < 0)
                 break;
         }
     }
-    fclose(file);
     if (err < 0)
         return err;
 
     octavesize = nnotes;
-    swap(octave, tmpoctave);
+    std::swap(octave, tmpoctave);
     synth->setAllPartMaps();
     return nnotes;
 }
@@ -487,15 +536,15 @@ int Microtonal::loadscl(string filename)
 // Loads the mapping from a kbm file
 int Microtonal::loadkbm(string filename)
 {
-    FILE *file = fopen(filename.c_str(), "r");
-    if (!file)
-        return -3; // last time we can return before fclose
-    char tmp[500];
+    string text = loadText(filename);
+    if (text == "")
+        return -3;
+    char tmp[BUFFSIZ];
+    size_t point = 0;
     int err = 0;
     int tmpMapSize;
-    fseek(file, 0, SEEK_SET);
     // loads the mapsize
-    if (loadline(file,&tmp[0]))
+    if (loadLine(text, point, tmp, BUFFSIZ))
         err = -4;
     else if (!sscanf(&tmp[0], "%d",&tmpMapSize))
         err = -2;
@@ -510,7 +559,7 @@ int Microtonal::loadkbm(string filename)
     if (err == 0)
     {
         // loads first MIDI note to retune
-        if (loadline(file, &tmp[0]))
+        if (loadLine(text, point, tmp, BUFFSIZ))
             err = -5;
         else if (!sscanf(&tmp[0], "%d", &tmpFirst))
             return -6;
@@ -522,7 +571,7 @@ int Microtonal::loadkbm(string filename)
     if (err == 0)
     {
         // loads last MIDI note to retune
-       if (loadline(file, &tmp[0]))
+       if (loadLine(text, point, tmp, BUFFSIZ))
             err = -5;
         else if (!sscanf(&tmp[0], "%d", &tmpLast))
             return -6;
@@ -534,7 +583,7 @@ int Microtonal::loadkbm(string filename)
     if (err == 0)
     {
         // loads the middle note where scale fro scale degree=0
-       if (loadline(file, &tmp[0]))
+       if (loadLine(text, point, tmp, BUFFSIZ))
             err = -5;
         else if (!sscanf(&tmp[0], "%d", &tmpMid))
             return -6;
@@ -546,7 +595,7 @@ int Microtonal::loadkbm(string filename)
     if (err == 0)
     {
         // loads the reference note
-       if (loadline(file, &tmp[0]))
+       if (loadLine(text, point, tmp, BUFFSIZ))
             err = -5;
         else if (!sscanf(&tmp[0], "%d", &tmpNote))
             return -6;
@@ -554,25 +603,25 @@ int Microtonal::loadkbm(string filename)
             err = -7;
     }
 
-    float tmpPAfreq;
+    float tmpPrefFreq;
     if (err == 0)
     {
         // loads the reference freq.
-        if (loadline(file, &tmp[0]))
+        if (loadLine(text, point, tmp, BUFFSIZ))
             err = -6;
         else
         {
 
-            if (!sscanf(&tmp[0], "%f", &tmpPAfreq))
+            if (!sscanf(&tmp[0], "%f", &tmpPrefFreq))
                 err = -6;
-            else if (tmpPAfreq < 1 || tmpPAfreq > 20000)
+            else if (tmpPrefFreq < 1 || tmpPrefFreq > 20000)
                 err = -8;
         }
     }
 
     // the scale degree(which is the octave) is not loaded
     // it is obtained by the tunings with getoctavesize() method
-    if (loadline(file, &tmp[0]))
+    if (loadLine(text, point, tmp, BUFFSIZ))
         err = -6;
 
     // load the mappings
@@ -582,7 +631,7 @@ int Microtonal::loadkbm(string filename)
     {
         for (int nline = 0; nline < tmpMapSize; ++nline)
         {
-            if (loadline(file, &tmp[0]))
+            if (loadLine(text, point, tmp, BUFFSIZ))
             {
                 err = -5;
                 break;
@@ -592,18 +641,17 @@ int Microtonal::loadkbm(string filename)
             tmpMap[nline] = x;
         }
     }
-    fclose(file);
     if (err < 0)
         return err;
 
     Pmappingenabled = 1;
     Pmapsize = tmpMapSize;
-    swap(Pmapping, tmpMap);
+    std::swap(Pmapping, tmpMap);
     Pfirstkey = tmpFirst;
     Plastkey = tmpLast;
     Pmiddlenote = tmpMid;
-    PAnote = tmpNote;
-    PAfreq = tmpPAfreq;
+    PrefNote = tmpNote;
+    PrefFreq = tmpPrefFreq;
     synth->setAllPartMaps();
     return tmpMapSize;
 }
@@ -620,8 +668,8 @@ void Microtonal::add2XML(XMLwrapper *xml)
     xml->addparbool("enabled", Penabled);
     xml->addpar("global_fine_detune", lrint(Pglobalfinedetune));
 
-    xml->addpar("a_note", PAnote);
-    xml->addparreal("a_freq", PAfreq);
+    xml->addpar("a_note", PrefNote);
+    xml->addparreal("a_freq", PrefFreq);
 
     if (!Penabled && xml->minimal)
         return;
@@ -634,7 +682,7 @@ void Microtonal::add2XML(XMLwrapper *xml)
 
         xml->beginbranch("OCTAVE");
         xml->addpar("octave_size", octavesize);
-        for (int i = 0; i < octavesize; ++i)
+        for (size_t i = 0; i < octavesize; ++i)
         {
             xml->beginbranch("DEGREE", i);
             if (octave[i].type == 1)
@@ -680,8 +728,8 @@ void Microtonal::getfromXML(XMLwrapper *xml)
     Penabled=xml->getparbool("enabled", Penabled);
     Pglobalfinedetune = xml->getpar127("global_fine_detune", Pglobalfinedetune);
 
-    PAnote = xml->getpar127("a_note", PAnote);
-    PAfreq = xml->getparreal("a_freq", PAfreq, 1.0, 10000.0);
+    PrefNote = xml->getpar127("a_note", PrefNote);
+    PrefFreq = xml->getparreal("a_freq", PrefFreq, 1.0, 10000.0);
 
     if (xml->enterbranch("SCALE"))
     {
@@ -693,7 +741,7 @@ void Microtonal::getfromXML(XMLwrapper *xml)
         if (xml->enterbranch("OCTAVE"))
         {
             octavesize = xml->getpar127("octave_size", octavesize);
-            for (int i = 0; i < octavesize; ++i)
+            for (size_t i = 0; i < octavesize; ++i)
             {
                 if (!xml->enterbranch("DEGREE", i))
                     continue;
@@ -749,7 +797,7 @@ void Microtonal::getfromXML(XMLwrapper *xml)
 
 bool Microtonal::saveXML(string filename)
 {
-    synth->getRuntime().xmlType = XML_MICROTONAL;
+    synth->getRuntime().xmlType = TOPLEVEL::XML::Scale;
     XMLwrapper *xml = new XMLwrapper(synth);
 
     xml->beginbranch("MICROTONAL");
@@ -778,6 +826,7 @@ bool Microtonal::loadXML(string filename)
     if (!xml->enterbranch("MICROTONAL"))
     {
         synth->getRuntime().Log(filename + " is not a scale file", 1);
+        delete xml;
         return false;
     }
     getfromXML(xml);
@@ -789,12 +838,11 @@ bool Microtonal::loadXML(string filename)
 
 float Microtonal::getLimits(CommandBlock *getData)
 {
-    float value = getData->data.value;
-    unsigned char type = getData->data.type;
+    float value = getData->data.value.F;
     int request = int(getData->data.type & TOPLEVEL::type::Default);
     int control = getData->data.control;
 
-    type &= (TOPLEVEL::source::MIDI | TOPLEVEL::source::CLI | TOPLEVEL::source::GUI); // source bits only
+    unsigned char type = 0;
 
     // microtonal defaults
     int min = 0;
@@ -805,13 +853,15 @@ float Microtonal::getLimits(CommandBlock *getData)
 
     switch (control)
     {
-        case SCALES::control::Afrequency:
-            min = 1.0f;
-            def = 440.0f;
-            max = 20000.0f;
+        case SCALES::control::refFrequency:
+            min = A_MIN;
+            def = A_DEF;
+            max = A_MAX;
             break;
-        case SCALES::control::Anote:
+        case SCALES::control::refNote:
+            min = 24;
             def = 69;
+            max = 84;
             type |= learnable;
             break;
         case SCALES::control::invertScale:
