@@ -4,7 +4,7 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2016-2018, Will Godfrey
+    Copyright 2016-2019, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -22,25 +22,25 @@
 
     This file is derivative of ZynAddSubFX original code.
 
-    Modified March 2018
+    Modified February 2019
 */
 
 #ifndef MICROTONAL_H
 #define MICROTONAL_H
 
+#include <cmath>
 #include <string>
-
-using namespace std;
-
-#include "Misc/MiscFuncs.h"
-
-class XMLwrapper;
-
-#define MAX_OCTAVE_SIZE 128
+#include "globals.h"
 
 class SynthEngine;
+class XMLwrapper;
 
-class Microtonal : private MiscFuncs
+using std::string;
+
+const size_t MAX_OCTAVE_SIZE = 128;
+
+
+class Microtonal
 {
     public:
         Microtonal(SynthEngine *_synth): synth(_synth) { defaults(); }
@@ -54,9 +54,9 @@ class Microtonal : private MiscFuncs
         unsigned char Pinvertupdown;
         int           Pinvertupdowncenter;
         unsigned char Penabled;
-        int           PAnote;
+        int           PrefNote;
         int           Pscaleshift;
-        float         PAfreq;
+        float         PrefFreq;
 
         // first and last key (to retune)
         int Pfirstkey;
@@ -74,7 +74,7 @@ class Microtonal : private MiscFuncs
         float Pglobalfinedetune;
 
         int getoctavesize(void);
-        void tuningtoline(int n, char *line, int maxn);
+        void tuningtoline(unsigned int n, char *line, int maxn);
         string tuningtotext(void);
         string keymaptotext(void);
         int loadscl(string filename); // load the tunings from a .scl file
@@ -94,19 +94,21 @@ class Microtonal : private MiscFuncs
         string reformatline(string text);
         bool validline(const char *line);
         int linetotunings(unsigned int nline, const char *line);
-        int loadline(FILE *file, char *line); // loads a line from the text file,
-                                              // ignoring the lines beginning with "!"
-        int octavesize;
+        int loadLine(string text, size_t &point, char *line, size_t maxlen);
+        // loads a line from the text file,
+        // ignoring the lines beginning with "!"
+        size_t octavesize;
 
-        struct {
+        struct Octave {
             unsigned char type; // 1 for cents or 2 for division
             double tuning;       // the real tuning (eg. +1.05946 for one halftone)
                                 // or 2.0 for one octave
             unsigned int x1; // the real tuning is x1 / x2
             unsigned int x2;
             string text;
-        } octave[MAX_OCTAVE_SIZE],
-          tmpoctave[MAX_OCTAVE_SIZE];
+        };
+        Octave octave[MAX_OCTAVE_SIZE];
+        Octave tmpoctave[MAX_OCTAVE_SIZE];
 
         float note_12et[128];
 
@@ -120,7 +122,7 @@ inline int Microtonal::getoctavesize(void)
 
 inline float Microtonal::getFixedNoteFreq(int note)
 {
-    return powf(2.0f, (float)(note - PAnote) / 12.0f) * PAfreq;
+    return powf(2.0f, (float)(note - PrefNote) / 12.0f) * PrefFreq;
 }
 
 
