@@ -2,7 +2,7 @@
     AlsaEngine.cpp
 
     Copyright 2009-2011, Alan Calvert
-    Copyright 2014-2018, Will Godfrey & others
+    Copyright 2014-2019, Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can
     redistribute it and/or modify it under the terms of the GNU General
@@ -17,18 +17,18 @@
     You should have received a copy of the GNU General Public License
     along with yoshimi.  If not, see <http://www.gnu.org/licenses/>.
 
-    Modified May 2018
 */
 
 #if defined(HAVE_ALSA)
 
-//#include <endian.h>
-
-using namespace std;
-
 #include "Misc/Config.h"
 #include "Misc/SynthEngine.h"
+#include "Misc/FormatFuncs.h"
 #include "MusicIO/AlsaEngine.h"
+#include <iostream>
+
+using func::asString;
+
 
 AlsaEngine::AlsaEngine(SynthEngine *_synth) :MusicIO(_synth)
 {
@@ -44,11 +44,7 @@ AlsaEngine::AlsaEngine(SynthEngine *_synth) :MusicIO(_synth)
     midi.handle = NULL;
     midi.alsaId = -1;
     midi.pThread = 0;
-#if __BYTE_ORDER  == __LITTLE_ENDIAN
-        little_endian = true;
-#else
-        little_endian = false;
-#endif
+    little_endian = synth->getRuntime().isLittleEndian;
 }
 
 
@@ -87,16 +83,49 @@ bail_out:
     return false;
 }
 
+std::string AlsaEngine::findMidiClients(snd_seq_t *seq)
+{
+    string result = "";
+    snd_seq_client_info_t *cinfo;
+    snd_seq_port_info_t *pinfo;
+
+    snd_seq_client_info_alloca(&cinfo);
+    snd_seq_port_info_alloca(&pinfo);
+    snd_seq_client_info_set_client(cinfo, -1);
+    while (snd_seq_query_next_client(seq, cinfo) >= 0) {
+        int client = snd_seq_client_info_get_client(cinfo);
+
+        if (client == SND_SEQ_CLIENT_SYSTEM)
+            continue; // don't show system timer and announce ports
+        snd_seq_port_info_set_client(pinfo, client);
+        snd_seq_port_info_set_port(pinfo, -1);
+        while (snd_seq_query_next_port(seq, pinfo) >= 0) {
+            // port must understand MIDI messages
+            if (!(snd_seq_port_info_get_type(pinfo)
+                  & SND_SEQ_PORT_TYPE_MIDI_GENERIC))
+                continue;
+            // we need both READ and SUBS_READ
+            if ((snd_seq_port_info_get_capability(pinfo)
+                 & (SND_SEQ_PORT_CAP_READ | SND_SEQ_PORT_CAP_SUBS_READ))
+                != (SND_SEQ_PORT_CAP_READ | SND_SEQ_PORT_CAP_SUBS_READ))
+                continue;
+            if (std::string(snd_seq_client_info_get_name(cinfo)) == "Midi Through")
+                continue; // don't want midi through
+            result = result + snd_seq_client_info_get_name(cinfo) + ":" + std::to_string(snd_seq_port_info_get_port(pinfo)) + ", ";
+        }
+    }
+    return result;
+}
 
 bool AlsaEngine::openMidi(void)
 {
-    midi.device = synth->getRuntime().midiDevice;
     const char* port_name = "input";
     int port_num;
     if (snd_seq_open(&midi.handle, "default", SND_SEQ_OPEN_INPUT, SND_SEQ_NONBLOCK) != 0)
     {
         synth->getRuntime().Log("Failed to open alsa midi");
-        goto bail_out;
+        Close();
+        return false;
     }
     snd_seq_client_info_t *seq_info;
     snd_seq_client_info_alloca(&seq_info);
@@ -127,26 +156,63 @@ bool AlsaEngine::openMidi(void)
     if (port_num < 0)
     {
         synth->getRuntime().Log("Failed to acquire alsa midi port");
-        goto bail_out;
+        Close();
+        return false;
     }
-    if (!midi.device.empty() && midi.device != "default")
-    {
-        bool midiSource = false;
-        if (snd_seq_parse_address(midi.handle,&midi.addr,midi.device.c_str()) == 0)
-        {
-            midiSource = (snd_seq_connect_from(midi.handle, port_num, midi.addr.client, midi.addr.port) == 0);
-        }
-        if (!midiSource)
-        {
-            synth->getRuntime().Log("Didn't find alsa MIDI source '" + midi.device + "'");
-            synth->getRuntime().midiDevice = "";
-        }
-    }
-    return true;
 
-bail_out:
-    Close();
-    return false;
+    std::string midilist;
+    switch(synth->getRuntime().alsaMidiType)
+    {
+        case 0:
+            midilist = synth->getRuntime().midiDevice;
+            break;
+        case 1:
+            midilist = findMidiClients(midi.handle);
+            break;
+        default:
+            synth->getRuntime().midiDevice = "";
+            return true;
+    }
+    std::string found = "";
+    if (!midilist.empty() && midilist != "default")
+    {
+        while (!midilist.empty())
+        {
+            string tmp;
+            while (midilist.find(' ') == 0 || midilist.find(',') == 0)
+                midilist.erase(0,1); // make entry clean
+            if (midilist.empty())
+                break;
+
+            size_t pos = midilist.find(',');
+            if (pos == std::string::npos)
+            {
+                tmp = midilist;
+                midilist = "";
+            }
+            else
+            {
+                tmp = midilist.substr(0, pos);
+                midilist = midilist.substr(pos + 1);
+            }
+            pos = tmp.find_last_not_of(' ');
+            tmp = tmp.substr(0, pos + 1);
+            midi.device = tmp;
+
+            bool midiSource = false;
+            if (snd_seq_parse_address(midi.handle,&midi.addr,midi.device.c_str()) == 0)
+            {
+                midiSource = (snd_seq_connect_from(midi.handle, port_num, midi.addr.client, midi.addr.port) == 0);
+            }
+            if (midiSource)
+                found += (", " + tmp);
+        }
+    }
+    if (found.substr(0, 2) == ", ")
+        synth->getRuntime().midiDevice = found.substr(2);
+    else
+        synth->getRuntime().midiDevice = "No MIDI sources seen";
+    return true;
 }
 
 
@@ -181,17 +247,17 @@ void AlsaEngine::Close(void)
 }
 
 
-string AlsaEngine::audioClientName(void)
+std::string AlsaEngine::audioClientName(void)
 {
-    string name = "yoshimi";
+    std::string name = "yoshimi";
     if (!synth->getRuntime().nameTag.empty())
         name += ("-" + synth->getRuntime().nameTag);
     return name;
 }
 
-string AlsaEngine::midiClientName(void)
+std::string AlsaEngine::midiClientName(void)
 {
-    string name = "yoshimi";
+    std::string name = "yoshimi";
     if (!synth->getRuntime().nameTag.empty())
         name += ("-" + synth->getRuntime().nameTag);
     //Andrew Deryabin: for multi-instance support add unique id to
@@ -232,7 +298,7 @@ bool AlsaEngine::prepHwparams(void)
         {SND_PCM_FORMAT_UNKNOWN, 0, false, true}
     };
     int formidx;
-    string formattxt = "";
+    std::string formattxt = "";
     card_chans = 2; // got to start somewhere
 
     unsigned int ask_samplerate = audio.samplerate;
@@ -273,17 +339,22 @@ bool AlsaEngine::prepHwparams(void)
     card_endian = card_formats[formidx].card_endian;
     card_signed = card_formats[formidx].card_signed;
 
-    synth->getRuntime().Log("March little endian = " + asString(little_endian), 2);
-
-    if (card_signed) // not currently used, may be later
-        formattxt = "Signed";
+    if (little_endian)
+        formattxt += "Little";
     else
-        formattxt = "Unsigned";
+        formattxt += "Big";
+
+    synth->getRuntime().Log("March is " + formattxt + " Endian", 2);
+
+    if (card_signed)
+        formattxt = "Signed ";
+    else
+        formattxt = "Unsigned ";
 
     if (card_endian)
-        formattxt += " Little";
+        formattxt += "Little";
     else
-        formattxt += " Big";
+        formattxt += "Big";
 
 
     alsaBad(snd_pcm_hw_params_set_rate_resample(audio.handle, hwparams, 1),
@@ -312,7 +383,7 @@ bool AlsaEngine::prepHwparams(void)
                 NULL), "failed to get period size"))
         goto bail_out;
 
-    synth->getRuntime().Log("Format = " + formattxt + " Endian " + asString(card_bits) +" Bit " + asString(card_chans) + " Channel" , 2);
+    synth->getRuntime().Log("Card Format is " + formattxt + " Endian " + asString(card_bits) +" Bit " + asString(card_chans) + " Channel" , 2);
     if (ask_buffersize != audio.period_size)
     {
         synth->getRuntime().Log("Asked for buffersize " + asString(ask_buffersize, 2)
@@ -420,34 +491,19 @@ void *AlsaEngine::AudioThread(void)
         audio.pcm_state = snd_pcm_state(audio.handle);
         if (audio.pcm_state != SND_PCM_STATE_RUNNING)
         {
-            bool done = false; // now done this way to suppress warnings
-            if (audio.pcm_state == SND_PCM_STATE_XRUN || audio.pcm_state == SND_PCM_STATE_SUSPENDED)
-            {
-                if (!xrunRecover())
-                    done = true;
-            }
-            if (!done || audio.pcm_state == SND_PCM_STATE_SETUP)
-            {
-                if (alsaBad(snd_pcm_prepare(audio.handle), "alsa audio pcm prepare failed"))
-                    done = true;
-            }
-            if (!done || audio.pcm_state == SND_PCM_STATE_PREPARED)
-            {
-                alsaBad(snd_pcm_start(audio.handle), "pcm start failed");
-            }
-            /*switch (audio.pcm_state)
+            switch (audio.pcm_state)
             {
                 case SND_PCM_STATE_XRUN:
 
                 case SND_PCM_STATE_SUSPENDED:
                     if (!xrunRecover())
                         break;
-                    // else fall through to ...
+                    /* falls through */
                 case SND_PCM_STATE_SETUP:
                     if (alsaBad(snd_pcm_prepare(audio.handle),
                                 "alsa audio pcm prepare failed"))
                         break;
-                    // and again
+                    /* falls through */
                 case SND_PCM_STATE_PREPARED:
                     alsaBad(snd_pcm_start(audio.handle), "pcm start failed");
                     break;
@@ -456,7 +512,7 @@ void *AlsaEngine::AudioThread(void)
                     synth->getRuntime().Log("Alsa AudioThread, weird SND_PCM_STATE: "
                                 + asString(audio.pcm_state));
                     break;
-            }*/
+            }
             audio.pcm_state = snd_pcm_state(audio.handle);
         }
         if (audio.pcm_state == SND_PCM_STATE_RUNNING)
@@ -556,7 +612,7 @@ bool AlsaEngine::xrunRecover(void)
             if (!alsaBad(snd_pcm_prepare(audio.handle), "pcm prepare failed"))
                 isgood = true;
         synth->getRuntime().Log("Alsa xrun recovery "
-                    + ((isgood) ? string("good") : string("not good")));
+                    + ((isgood) ? std::string("good") : std::string("not good")));
     }
     return isgood;
 }
@@ -565,13 +621,13 @@ bool AlsaEngine::xrunRecover(void)
 bool AlsaEngine::Start(void)
 {
     if (NULL != midi.handle && !synth->getRuntime().startThread(&midi.pThread, _MidiThread,
-                                                    this, true, 1, false, "Alsa midi"))
+                                                    this, true, 1, "Alsa midi"))
     {
         synth->getRuntime().Log("Failed to start Alsa midi thread");
         goto bail_out;
     }
     if (NULL != audio.handle && !synth->getRuntime().startThread(&audio.pThread, _AudioThread,
-                                                     this, true, 0, false, "Alsa audio"))
+                                                     this, true, 0, "Alsa audio"))
     {
         synth->getRuntime().Log(" Failed to start Alsa audio thread");
         goto bail_out;
@@ -702,12 +758,12 @@ void *AlsaEngine::MidiThread(void)
 }
 
 
-bool AlsaEngine::alsaBad(int op_result, string err_msg)
+bool AlsaEngine::alsaBad(int op_result, std::string err_msg)
 {
     bool isbad = (op_result < 0);
     if (isbad)
         synth->getRuntime().Log("Error, alsa audio: " +err_msg + ": "
-                     + string(snd_strerror(op_result)));
+                     + std::string(snd_strerror(op_result)));
     return isbad;
 }
 
