@@ -2,7 +2,7 @@
     JackEngine.cpp
 
     Copyright 2009-2011, Alan Calvert
-    Copyright 2014-2018, Will Godfrey & others
+    Copyright 2014-2019, Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can
     redistribute it and/or modify it under the terms of the GNU General
@@ -17,23 +17,23 @@
     You should have received a copy of the GNU General Public License
     along with yoshimi.  If not, see <http://www.gnu.org/licenses/>.
 
-    Modified May 2018
 */
 
 #include <errno.h>
-#include <unistd.h>
+#include <iostream>
+
 #include <jack/midiport.h>
 #include <jack/thread.h>
-#include <fcntl.h>
-#include <sys/stat.h>
-#include <sstream>
-#include <stdio.h>
-
-using namespace std;
+#include <unistd.h>     // for usleep()
 
 #include "Misc/Config.h"
+#include "Misc/FormatFuncs.h"
 #include "MusicIO/JackEngine.h"
 
+using func::asString;
+using func::asHexString;
+
+
 JackEngine::JackEngine(SynthEngine *_synth) : MusicIO(_synth), jackClient(NULL)
 {
     audio.jackSamplerate = 0;
@@ -47,7 +47,7 @@ JackEngine::JackEngine(SynthEngine *_synth) : MusicIO(_synth), jackClient(NULL)
 }
 
 
-bool JackEngine::connectServer(string server)
+bool JackEngine::connectServer(std::string server)
 {
     for (int tries = 0; tries < 3 && !jackClient; ++tries)
     {
@@ -72,11 +72,11 @@ bool JackEngine::connectServer(string server)
 }
 
 
-bool JackEngine::openJackClient(string server)
+bool JackEngine::openJackClient(std::string server)
 {
     int jopts = JackNullOption;
     jack_status_t jstatus;
-    string clientname = "yoshimi";
+    std::string clientname = "yoshimi";
     if (synth->getRuntime().nameTag.size())
         clientname += ("-" + synth->getRuntime().nameTag);
 
@@ -90,7 +90,7 @@ bool JackEngine::openJackClient(string server)
         snprintf(sUniqueId, sizeof(sUniqueId), "%d", synthUniqueId);
         clientname += ("-" + std::string(sUniqueId));
     }
-
+    //std::cout << " C name " << clientname << std::endl;
     bool named_server = server.size() > 0 && server.compare("default") != 0;
     if (named_server)
         jopts |= JackServerName;
@@ -126,7 +126,7 @@ bool JackEngine::openJackClient(string server)
     if (jackClient)
         return true;
     else
-        synth->getRuntime().Log("Failed jack_client_open(), status: " + synth->getRuntime().asHexString((int)jstatus), 1);
+        synth->getRuntime().Log("Failed jack_client_open(), status: " + asHexString((int)jstatus), 1);
     return false;
 }
 
@@ -167,17 +167,24 @@ bool JackEngine::Start(void)
         synth->getRuntime().Log("Failed to activate jack client");
         goto bail_out;
     }
+    /*
+     * TODO fix this - now moved to where it should be.
+     * Shows identical results but doesn't connect.
+     * Original 1.4.1 version also fails - it used to work.
+     */
 
     // style-wise I think the next bit is the wrong place
-    if (synth->getRuntime().midiEngine  == jack_midi
-      && synth->getRuntime().midiDevice.size()
-      && synth->getRuntime().midiDevice != "default"
-      && jack_connect(jackClient,synth->getRuntime().midiDevice.c_str(),jack_port_name(midi.port)))
+    /*if (synth->getRuntime().midiEngine  == jack_midi
+      && !synth->getRuntime().midiDevice.empty()
+      && synth->getRuntime().midiDevice != "default")
     {
-        synth->getRuntime().Log("Didn't find jack MIDI source '"
-        + synth->getRuntime().midiDevice + "'", 1);
-        synth->getRuntime().midiDevice = "";
-    }
+        if(jack_connect(jackClient, synth->getRuntime().midiDevice.c_str(), jack_port_name(midi.port)))
+        {
+            synth->getRuntime().Log("Didn't find jack MIDI source '"
+            + synth->getRuntime().midiDevice + "'", 1);
+            synth->getRuntime().midiDevice = "";
+        }
+    }*/
     return true;
 
 bail_out:
@@ -231,7 +238,7 @@ void JackEngine::registerAudioPort(int partnum)
          * individual ports (at startup) if part is not configured for
          * direct O/P.
          */
-        string portName;
+        std::string portName;
         if(synth->part [partnum] && synth->partonoffRead(partnum) && (synth->part [partnum]->Paudiodest > 1))
         {
             portName = "track_" + asString(partnum + 1) + "_l";
@@ -297,6 +304,15 @@ bool JackEngine::openMidi(void)
         synth->getRuntime().Log("Failed to register jack midi port");
         return false;
     }
+
+    std::cout << "client " << jackClient<< "  device " << synth->getRuntime().midiDevice << "  port " << jack_port_name(midi.port) << std::endl;
+    if(jack_connect(jackClient, synth->getRuntime().midiDevice.c_str(), jack_port_name(midi.port)))
+        {
+            synth->getRuntime().Log("Didn't find jack MIDI source '"
+            + synth->getRuntime().midiDevice + "'");
+            //synth->getRuntime().midiDevice = "";
+        }
+
     return true;
 }
 
@@ -318,14 +334,14 @@ bool JackEngine::connectJackPorts(void)
         {
             if(ret == EEXIST)
             {
-            synth->getRuntime().Log(string(port_name)
-                        + " is already connected to jack port " + string(playback_ports[port])
+            synth->getRuntime().Log(std::string(port_name)
+                        + " is already connected to jack port " + std::string(playback_ports[port])
                         + ", status " + asString(ret));
             }
             else
             {
-            synth->getRuntime().Log("Cannot connect " + string(port_name)
-                        + " to jack port " + string(playback_ports[port])
+            synth->getRuntime().Log("Cannot connect " + std::string(port_name)
+                        + " to jack port " + std::string(playback_ports[port])
                         + ", status " + asString(ret));
             return false;
             }
@@ -344,13 +360,13 @@ int JackEngine::clientId(void)
 }
 
 
-string JackEngine::clientName(void)
+std::string JackEngine::clientName(void)
 {
     if (jackClient)
-        return string(jack_get_client_name(jackClient));
+        return std::string(jack_get_client_name(jackClient));
     else
         synth->getRuntime().Log("clientName() with null jackClient");
-    return string("Oh, yoshimi :-(");
+    return std::string("Oh, yoshimi :-(");
 }
 
 
@@ -476,7 +492,10 @@ bool JackEngine::processMidi(jack_nframes_t nframes)
         if(!jack_midi_event_get(&jEvent, portBuf, idx))
         {
             if (jEvent.size >= 1 && jEvent.size <= 4) // no interest in zero sized or long events
+            {
+                //std::cout << "Offset " << int(jEvent.time) << std::endl;
                 setMidi(jEvent.buffer[0], jEvent.buffer[1], jEvent.buffer[2]);
+            }
         }
     }
     return true;
@@ -525,12 +544,12 @@ void JackEngine::_jsessionCallback(jack_session_event_t *event, void *arg)
 
 void JackEngine::jsessionCallback(jack_session_event_t *event)
 {
-    string uuid = string(event->client_uuid);
-    string filename = string("yoshimi-") + uuid + string(".xml");
-    string filepath = string(event->session_dir) + filename;
+    std::string uuid = std::string(event->client_uuid);
+    std::string filename = std::string("yoshimi-") + uuid + std::string(".xml");
+    std::string filepath = std::string(event->session_dir) + filename;
     synth->getRuntime().setJackSessionSave((int)event->type, filepath);
-    string cmd = synth->getRuntime().programCmd() + string(" -U ") + uuid
-                 + string(" -u ${SESSION_DIR}") + filename;
+    std::string cmd = synth->getRuntime().programCmd() + std::string(" -U ") + uuid
+                 + std::string(" -u ${SESSION_DIR}") + filename;
     event->command_line = strdup(cmd.c_str());
     if (jack_session_reply(jackClient, event))
         synth->getRuntime().Log("Jack session reply failed");
