@@ -4,7 +4,8 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2017-2018 Will Godfrey
+    Copyright 2017-2019 Will Godfrey
+    Copyright 2020 Kristian Amlie, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -21,7 +22,6 @@
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
     This file is derivative of ZynAddSubFX original code.
-    Modified August 2018
 */
 
 #include <iostream>
@@ -40,7 +40,7 @@ ADnoteParameters::ADnoteParameters(FFTwrapper *fft_, SynthEngine *_synth) :
     Presets(_synth),
     fft(fft_)
 {
-    setpresettype("ADnoteParameters");
+    setpresettype("Padsyth");
     GlobalPar.FreqEnvelope = new EnvelopeParams(0, 0, synth);
     GlobalPar.FreqEnvelope->ASRinit(64, 50, 64, 60);
     GlobalPar.FreqLfo = new LFOParams(70, 0, 64, 0, 0, 0, 0, 0, synth);
@@ -145,10 +145,12 @@ void ADnoteParameters::defaults(int n)
     VoicePar[nvoice].PFMFixedFreq = false;
 
     // I use the internal oscillator (-1)
+    VoicePar[nvoice].PVoice = -1;
     VoicePar[nvoice].PFMVoice = -1;
 
     VoicePar[nvoice].PFMVolume = 90;
     VoicePar[nvoice].PFMVolumeDamp = 64;
+    VoicePar[nvoice].PFMDetuneFromBaseOsc = 1;
     VoicePar[nvoice].PFMDetune = 8192;
     VoicePar[nvoice].PFMCoarseDetune = 0;
     VoicePar[nvoice].PFMDetuneType = 0;
@@ -156,8 +158,8 @@ void ADnoteParameters::defaults(int n)
     VoicePar[nvoice].PFMAmpEnvelopeEnabled = 0;
     VoicePar[nvoice].PFMVelocityScaleFunction = 64;
 
-    VoicePar[nvoice].OscilSmp->defaults();
-    VoicePar[nvoice].FMSmp->defaults();
+    VoicePar[nvoice].POscil->defaults();
+    VoicePar[nvoice].POscilFM->defaults();
 
     VoicePar[nvoice].AmpEnvelope->defaults();
     VoicePar[nvoice].AmpLfo->defaults();
@@ -177,8 +179,10 @@ void ADnoteParameters::defaults(int n)
 // Init the voice parameters
 void ADnoteParameters::enableVoice(int nvoice)
 {
-    VoicePar[nvoice].OscilSmp = new OscilGen(fft, GlobalPar.Reson, synth);
-    VoicePar[nvoice].FMSmp = new OscilGen(fft, NULL, synth);
+    VoicePar[nvoice].POscil = new OscilParameters(synth);
+    VoicePar[nvoice].POscilFM = new OscilParameters(synth);
+    VoicePar[nvoice].OscilSmp = new OscilGen(fft, GlobalPar.Reson, synth, VoicePar[nvoice].POscil);
+    VoicePar[nvoice].FMSmp = new OscilGen(fft, NULL, synth, VoicePar[nvoice].POscilFM);
 
     VoicePar[nvoice].AmpEnvelope = new EnvelopeParams(64, 1, synth);
     VoicePar[nvoice].AmpEnvelope->ADSRinit_dB(0, 100, 127, 100);
@@ -223,6 +227,8 @@ void ADnoteParameters::killVoice(int nvoice)
 {
     delete VoicePar[nvoice].OscilSmp;
     delete VoicePar[nvoice].FMSmp;
+    delete VoicePar[nvoice].POscil;
+    delete VoicePar[nvoice].POscilFM;
 
     delete VoicePar[nvoice].AmpEnvelope;
     delete VoicePar[nvoice].AmpLfo;
@@ -316,6 +322,7 @@ void ADnoteParameters::add2XMLsection(XMLwrapper *xml, int n)
     xml->addpar("delay", VoicePar[nvoice].PDelay);
     xml->addparbool("resonance", VoicePar[nvoice].Presonance);
 
+    xml->addpar("input_voice", VoicePar[nvoice].PVoice);
     xml->addpar("ext_oscil", VoicePar[nvoice].Pextoscil);
     xml->addpar("ext_fm_oscil", VoicePar[nvoice].PextFMoscil);
 
@@ -328,7 +335,7 @@ void ADnoteParameters::add2XMLsection(XMLwrapper *xml, int n)
     xml->addpar("fm_enabled", VoicePar[nvoice].PFMEnabled);
 
     xml->beginbranch("OSCIL");
-        VoicePar[nvoice].OscilSmp->add2XML(xml);
+        VoicePar[nvoice].POscil->add2XML(xml);
     xml->endbranch();
 
 
@@ -422,6 +429,7 @@ void ADnoteParameters::add2XMLsection(XMLwrapper *xml, int n)
                 xml->endbranch();
             }
             xml->beginbranch("MODULATOR");
+                xml->addparbool("detune_from_base_osc", VoicePar[nvoice].PFMDetuneFromBaseOsc);
                 xml->addpar("detune", VoicePar[nvoice].PFMDetune);
                 xml->addpar("coarse_detune", VoicePar[nvoice].PFMCoarseDetune);
                 xml->addpar("detune_type", VoicePar[nvoice].PFMDetuneType);
@@ -436,7 +444,7 @@ void ADnoteParameters::add2XMLsection(XMLwrapper *xml, int n)
                 }
 
                 xml->beginbranch("OSCIL");
-                    VoicePar[nvoice].FMSmp->add2XML(xml);
+                    VoicePar[nvoice].POscilFM->add2XML(xml);
                 xml->endbranch();
 
             xml->endbranch();
@@ -640,6 +648,8 @@ void ADnoteParameters::getfromXMLsection(XMLwrapper *xml, int n)
     VoicePar[nvoice].PDelay = xml->getpar127("delay", VoicePar[nvoice].PDelay);
     VoicePar[nvoice].Presonance = xml->getparbool("resonance", VoicePar[nvoice].Presonance);
 
+    VoicePar[nvoice].PVoice =
+        xml->getpar("input_voice", VoicePar[nvoice].PVoice, -1, nvoice - 1);
     VoicePar[nvoice].Pextoscil = xml->getpar("ext_oscil", -1, -1, nvoice - 1);
     VoicePar[nvoice].PextFMoscil = xml->getpar("ext_fm_oscil", -1, -1,nvoice - 1);
 
@@ -657,7 +667,7 @@ void ADnoteParameters::getfromXMLsection(XMLwrapper *xml, int n)
 
     if (xml->enterbranch("OSCIL"))
     {
-        VoicePar[nvoice].OscilSmp->getfromXML(xml);
+        VoicePar[nvoice].POscil->getfromXML(xml);
         xml->exitbranch();
     }
 
@@ -780,26 +790,60 @@ void ADnoteParameters::getfromXMLsection(XMLwrapper *xml, int n)
 
         if (xml->enterbranch("MODULATOR"))
         {
-            VoicePar[nvoice].PFMDetune =
-                xml->getpar("detune",VoicePar[nvoice].PFMDetune, 0, 16383);
-            VoicePar[nvoice].PFMCoarseDetune =
-                xml->getpar("coarse_detune", VoicePar[nvoice].PFMCoarseDetune, 0, 16383);
-            VoicePar[nvoice].PFMDetuneType =
-                xml->getpar127("detune_type", VoicePar[nvoice].PFMDetuneType);
+            bool loadFMFreqParams = true;
+            VoicePar[nvoice].PFMDetuneFromBaseOsc =
+                xml->getparbool("detune_from_base_osc", 127);
+            if (VoicePar[nvoice].PFMDetuneFromBaseOsc == 127) {
+                // In the past it was not possible to choose whether to include
+                // detuning from the base oscillator. For local modulators it
+                // was always enabled, for imported voice modulators it was
+                // always disabled. To load old patches correctly, we apply this
+                // old behavior here if the XML element is missing from the
+                // patch. New patches will always save one or the other.
+                //
+                // In a similar fashion, it was not possible to apply frequency
+                // parameters to imported voice modulators in the past, however
+                // it was possible to save them if you edited them before
+                // switching to an imported voice. Now that frequency parameters
+                // are respected, we need to ignore those parameters for old
+                // instruments that saved them, but didn't use them, otherwise
+                // the instrument will sound different.
+                if (VoicePar[nvoice].PFMVoice >= 0) {
+                    VoicePar[nvoice].PFMDetuneFromBaseOsc = 0;
+                    loadFMFreqParams = false;
 
-            VoicePar[nvoice].PFMFreqEnvelopeEnabled =
-                xml->getparbool("freq_envelope_enabled",
-                                VoicePar[nvoice].PFMFreqEnvelopeEnabled);
-            VoicePar[nvoice].PFMFixedFreq = xml->getparbool("fixed_freq", VoicePar[nvoice].PFMFixedFreq);
-            if (xml->enterbranch("FREQUENCY_ENVELOPE"))
-            {
-                VoicePar[nvoice].FMFreqEnvelope->getfromXML(xml);
-                xml->exitbranch();
+                    // In the past the fixed frequency of the imported voice was
+                    // respected. Now, the fixed frequency of the modulator is
+                    // respected. So if we load an old patch, fetch that setting
+                    // from the imported voice.
+                    VoicePar[nvoice].PFMFixedFreq =
+                        VoicePar[VoicePar[nvoice].PFMVoice].Pfixedfreq;
+                } else {
+                    VoicePar[nvoice].PFMDetuneFromBaseOsc = 1;
+                }
+            }
+            if (loadFMFreqParams) {
+                VoicePar[nvoice].PFMDetune =
+                    xml->getpar("detune",VoicePar[nvoice].PFMDetune, 0, 16383);
+                VoicePar[nvoice].PFMCoarseDetune =
+                    xml->getpar("coarse_detune", VoicePar[nvoice].PFMCoarseDetune, 0, 16383);
+                VoicePar[nvoice].PFMDetuneType =
+                    xml->getpar127("detune_type", VoicePar[nvoice].PFMDetuneType);
+
+                VoicePar[nvoice].PFMFreqEnvelopeEnabled =
+                    xml->getparbool("freq_envelope_enabled",
+                                    VoicePar[nvoice].PFMFreqEnvelopeEnabled);
+                VoicePar[nvoice].PFMFixedFreq = xml->getparbool("fixed_freq", VoicePar[nvoice].PFMFixedFreq);
+                if (xml->enterbranch("FREQUENCY_ENVELOPE"))
+                {
+                    VoicePar[nvoice].FMFreqEnvelope->getfromXML(xml);
+                    xml->exitbranch();
+                }
             }
 
             if (xml->enterbranch("OSCIL"))
             {
-                VoicePar[nvoice].FMSmp->getfromXML(xml);
+                VoicePar[nvoice].POscilFM->getfromXML(xml);
                 xml->exitbranch();
             }
 
@@ -812,13 +856,12 @@ void ADnoteParameters::getfromXMLsection(XMLwrapper *xml, int n)
 
 float ADnoteParameters::getLimits(CommandBlock *getData)
 {
-    float value = getData->data.value;
-    unsigned char type = getData->data.type;
-    int request = type & TOPLEVEL::type::Default;
+    float value = getData->data.value.F;
+    int request = int(getData->data.type & TOPLEVEL::type::Default);
     int control = getData->data.control;
     int engine = getData->data.engine;
 
-    type &= (TOPLEVEL::source::MIDI | TOPLEVEL::source::CLI | TOPLEVEL::source::GUI); // source bits only
+    unsigned char type = 0;
 
     // addnote defaults
     int min = 0;
@@ -865,7 +908,8 @@ float ADnoteParameters::getLimits(CommandBlock *getData)
                 break;
 
             case ADDSYNTH::control::detuneType:
-                max = 3;
+                min = 1;
+                max = 4;
                 break;
 
             case ADDSYNTH::control::coarseDetune:
@@ -976,6 +1020,12 @@ float ADnoteParameters::getLimits(CommandBlock *getData)
             max = 6;
             break;
 
+        case ADDVOICE::control::externalOscillator:
+            min = -1;
+            def = -1;
+            max = 6;
+            break;
+
         case ADDVOICE::control::detuneFrequency:
             type |= learnable;
             min = -8192;
@@ -1104,6 +1154,10 @@ float ADnoteParameters::getLimits(CommandBlock *getData)
             min = -8192;
             max = 8191;
             break;
+        case ADDVOICE::control::modulatorDetuneFromBaseOsc:
+            def = 1;
+            max = 1;
+            break;
         case ADDVOICE::control::modulatorFrequencyAs440Hz:
             max = 1;
             break;
@@ -1160,7 +1214,7 @@ float ADnoteParameters::getLimits(CommandBlock *getData)
             break;
 
         case ADDVOICE::control::soundType:
-            max = 2;
+            max = 3;
             break;
 
         default:
@@ -1191,21 +1245,3 @@ float ADnoteParameters::getLimits(CommandBlock *getData)
     }
     return value;
 }
-
-void ADnoteParameters::postrender(void)
-{
-    // loop over our gathered dirty flags and unset them for the next period
-      GlobalPar.AmpLfo->updated
-    = GlobalPar.FilterLfo->updated
-    = GlobalPar.FreqLfo->updated
-    = false;
-    for (int i = 0; i < NUM_VOICES; ++i)
-    {
-        if (VoicePar[i].Enabled)
-              VoicePar[i].AmpLfo->updated
-            = VoicePar[i].FilterLfo->updated
-            = VoicePar[i].FreqLfo->updated
-            = false;
-
-    }
-}
