@@ -5,6 +5,7 @@
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
     Copyright 2017-2018, Will Godfrey
+    Copyright 2020 Kristian Amlie
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -21,7 +22,7 @@
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
     This file is derivative of ZynAddSubFX original code.
-    Modified February 2018
+
 */
 
 #ifndef AD_NOTE_PARAMETERS_H
@@ -30,8 +31,9 @@
 #include "Params/EnvelopeParams.h"
 #include "Params/LFOParams.h"
 #include "Params/FilterParams.h"
-#include "Synth/OscilGen.h"
+#include "Params/OscilParameters.h"
 #include "Synth/Resonance.h"
+#include "Synth/OscilGen.h"
 #include "Misc/XMLwrapper.h"
 #include "DSP/FFTwrapper.h"
 #include "Params/Presets.h"
@@ -101,12 +103,13 @@ struct ADnoteVoiceParam { // Voice parameters
     unsigned char PDelay;                   // Voice Delay
     unsigned char Presonance;               // If resonance is enabled for this voice
     short int     Pextoscil,                // What external oscil should I use,
-                  PextFMoscil;              // -1 for internal OscilSmp & FMSmp
+                  PextFMoscil;              // -1 for internal POscil & POscilFM
                                             // it is not allowed that the externoscil,
                                             // externFMoscil => current voice
     unsigned char Poscilphase, PFMoscilphase; // oscillator phases
     unsigned char Pfilterbypass;            // filter bypass
-    OscilGen     *OscilSmp;
+    OscilParameters *POscil;
+    OscilGen        *OscilSmp;
 
     // Frequency parameters
     unsigned char Pfixedfreq;   // If the base frequency is fixed to 440 Hz
@@ -159,16 +162,22 @@ struct ADnoteVoiceParam { // Voice parameters
     unsigned char PFilterVelocityScaleFunction;
 
 
+    short int     PVoice;     // Voice that I use as external oscillator.
+                              // It is -1 if I use POscil(default).
+                              // It may not be equal or bigger than current voice
+
     // Modullator parameters
     unsigned char PFMEnabled; // 0 = off, 1 = Morph, 2 = RM, 3 = PM, 4 = FM, 5 = PWM
-    short int     PFMVoice;   // Voice that I use as modullator instead of FMSmp.
-                              // It is -1 if I use FMSmp(default).
+    short int     PFMVoice;   // Voice that I use as modullator instead of POscilFM.
+                              // It is -1 if I use POscilFM(default).
                               // It may not be equal or bigger than current voice
-    OscilGen *FMSmp;          // Modullator oscillator
+    OscilParameters *POscilFM;// Modullator oscillator
+    OscilGen        *FMSmp;
 
     unsigned char      PFMVolume;                // Modulator Volume
     unsigned char      PFMVolumeDamp;            // Modulator damping at higher frequencies
     unsigned char      PFMVelocityScaleFunction; // Modulator Velocity Sensing
+    unsigned char      PFMDetuneFromBaseOsc;     // Whether we inherit the base oscillator's detuning
     unsigned short int PFMDetune;                // Fine Detune of the Modulator
     unsigned short int PFMCoarseDetune;          // Coarse Detune of the Modulator
     unsigned char      PFMDetuneType;            // The detune type
@@ -198,7 +207,6 @@ class ADnoteParameters : public Presets
         ADnoteGlobalParam GlobalPar;
         ADnoteVoiceParam VoicePar[NUM_VOICES];
         static int ADnote_unison_sizes[15];
-        void postrender(void);
 
     private:
         void defaults(int n); // n is the nvoice
