@@ -4,7 +4,8 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2017-2018, Will Godfrey
+    Copyright 2017-2019, Will Godfrey
+    Copyright 2020 Kristian Amlie
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -22,12 +23,13 @@
 
     This file is derivative of original ZynAddSubFX code.
 
-    Modified July 2018
 */
 
 #ifndef LFO_PARAMS_H
 #define LFO_PARAMS_H
 
+#include <sys/types.h>
+
 #include "Misc/XMLwrapper.h"
 #include "Params/Presets.h"
 
@@ -43,19 +45,19 @@ class LFOParams : public Presets
 
         void add2XML(XMLwrapper *xml);
         void defaults(void);
+        void setPfreq(int32_t n);
         void getfromXML(XMLwrapper *xml);
-
-        void setPfreq(float n) { Pfreq = n; updated = true; }
-        void setPintensity(unsigned char n) { Pintensity = n; updated = true; }
+        void setPintensity(unsigned char n) { Pintensity = n; presetsUpdated(); }
         void setPstartphase(unsigned char n) { Pstartphase = n; }
-        void setPLFOtype(unsigned char n) { PLFOtype = n; updated = true; }
-        void setPrandomness(unsigned char n) { Prandomness = n; updated = true; }
-        void setPfreqrand(unsigned char n) { Pfreqrand = n; updated = true; }
+        void setPLFOtype(unsigned char n) { PLFOtype = n; presetsUpdated(); }
+        void setPrandomness(unsigned char n) { Prandomness = n; presetsUpdated(); }
+        void setPfreqrand(unsigned char n) { Pfreqrand = n; presetsUpdated(); }
         void setPdelay(unsigned char n) { Pdelay = n; }
-        void setPcontinous(unsigned char n) { Pcontinous = n; updated = true; }
-        void setPstretch(unsigned char n) { Pstretch = n; updated = true; }
+        void setPcontinous(unsigned char n) { Pcontinous = n; presetsUpdated(); }
+        void setPstretch(unsigned char n) { Pstretch = n; presetsUpdated(); }
 
         // MIDI Parameters
+        int32_t PfreqI;
         float Pfreq;
         unsigned char Pintensity;
         unsigned char Pstartphase;
@@ -68,7 +70,6 @@ class LFOParams : public Presets
 
         int fel;         // kind of LFO - 0 frequency, 1 amplitude, 2 filter
        // static int time; // used by Pcontinous - moved to SynthEngine to make it per-instance
-        bool updated;
 
     private:
         // Default parameters
