@@ -4,7 +4,8 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2017-2018 Will Godfrey
+    Copyright 2017-2019 Will Godfrey
+    Copyright 2020 Kristian Amlie
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -22,14 +23,8 @@
 
     This file is a derivative of a ZynAddSubFX original.
 
-    Modified October 2018
 */
 
-#include <cmath>
-#include <iostream>
-
-using namespace std;
-
 #include "Misc/XMLwrapper.h"
 #include "DSP/FFTwrapper.h"
 #include "Synth/OscilGen.h"
@@ -38,17 +33,22 @@ using namespace std;
 #include "Params/LFOParams.h"
 #include "Params/FilterParams.h"
 #include "Misc/SynthEngine.h"
+#include "Misc/FileMgrFuncs.h"
 #include "Params/PADnoteParameters.h"
 #include "Misc/WavFile.h"
 
+using file::saveData;
+
+
 PADnoteParameters::PADnoteParameters(FFTwrapper *fft_, SynthEngine *_synth) : Presets(_synth)
 {
-    setpresettype("PADnoteParameters");
+    setpresettype("Ppadsyth");
     fft = fft_;
 
     resonance = new Resonance(synth);
-    oscilgen = new OscilGen(fft_, resonance, synth);
-    oscilgen->ADvsPAD = true;
+    POscil = new OscilParameters(synth);
+    POscil->ADvsPAD = true;
+    oscilgen = new OscilGen(fft_, resonance, synth, POscil);
 
     FreqEnvelope = new EnvelopeParams(0, 0, synth);
     FreqEnvelope->ASRinit(64, 50, 64, 60);
@@ -74,6 +74,7 @@ PADnoteParameters::~PADnoteParameters()
 {
     deletesamples();
     delete oscilgen;
+    delete POscil;
     delete resonance;
     delete FreqEnvelope;
     delete FreqLfo;
@@ -297,7 +298,6 @@ float PADnoteParameters::getprofile(float *smp, int size)
     float max = 0.0f;
     for (int i = 0; i < size; ++i)
     {
-        smp[i] = smp[i];
         if (smp[i] > max)
             max = smp[i];
     }
@@ -346,14 +346,7 @@ float PADnoteParameters::getNhr(int n)
     switch (Phrpos.type)
     {
     case 1:
-        FR2Z2I(par2 * par2 * 100.0f, thresh);
-        thresh += 1;
-        /*
-         * leaving many of the old versions here
-         * as there may be subtle effects of adding
-         * or subtracting after the conversion
-         */
-        //thresh = (int)truncf(par2 * par2 * 100.0f) + 1;
+        thresh = int(par2 * par2 * 100.0f) + 1;
         if (n < thresh)
             result = n;
         else
@@ -361,9 +354,7 @@ float PADnoteParameters::getNhr(int n)
         break;
 
     case 2:
-        FR2Z2I(par2 * par2 * 100.0f, thresh);
-        thresh += 1;
-        //thresh = (int)truncf(par2 * par2 * 100.0f) + 1;
+        thresh = int(par2 * par2 * 100.0f) + 1;
         if (n < thresh)
             result = n;
         else
@@ -481,25 +472,17 @@ void PADnoteParameters::generatespectrum_bandwidthMode(float *spectrum,
             break;
         }
         bw = bw * powf(realfreq / basefreq, power);
-        int ibw;
-        FR2Z2I(bw / (synth->samplerate_f * 0.5f) * size, ibw);
-        ibw += 1;
-        //int ibw = (int)truncf((bw / (synth->samplerate_f * 0.5f) * size)) + 1;
+        int ibw = int((bw / (synth->samplerate_f * 0.5f) * size)) + 1;
         float amp = harmonics[nh - 1];
         if (resonance->Penabled)
             amp *= resonance->getfreqresponse(realfreq);
         if (ibw > profilesize)
         {   // if the bandwidth is larger than the profilesize
             float rap = sqrtf((float)profilesize / (float)ibw);
-            int cfreq;
-            FR2Z2I(realfreq / (synth->halfsamplerate_f) * size, cfreq);
-            cfreq -= (ibw / 2);
-            //int cfreq = (int)truncf(realfreq / (synth->halfsamplerate_f) * size) - ibw / 2;
+            int cfreq = int(realfreq / (synth->halfsamplerate_f) * size) - ibw / 2;
             for (int i = 0; i < ibw; ++i)
             {
-                int src;
-                FR2Z2I(i * rap * rap, src);
-                //int src = (int)truncf(i * rap * rap);
+                int src = int(i * rap * rap);
                 int spfreq = i + cfreq;
                 if (spfreq < 0)
                     continue;
@@ -571,9 +554,7 @@ void PADnoteParameters::generatespectrum_otherModes(float *spectrum,
         float amp = harmonics[nh - 1];
         if (resonance->Penabled)
             amp *= resonance->getfreqresponse(realfreq);
-        int cfreq;
-        FR2Z2I(realfreq / (synth->halfsamplerate_f) * size, cfreq);
-        //int cfreq = (int)truncf(realfreq / (synth->halfsamplerate_f) * size);
+        int cfreq = int(realfreq / (synth->halfsamplerate_f) * size);
         spectrum[cfreq] = amp + 1e-9f;
     }
 
@@ -709,9 +690,14 @@ void PADnoteParameters::setPan(char pan)
 }
 
 
-// Ported from ZynAddSubFX V 2.4.4
 bool PADnoteParameters::export2wav(std::string basefilename)
 {
+    string type;
+    if (synth->getRuntime().isLittleEndian)
+        type = "RIFF"; // default wave format
+    else
+        type = "RIFX";
+
     basefilename += "_PADsynth_";
     bool isOK = true;
     for(int k = 0; k < PAD_MAX_SAMPLES; ++k)
@@ -720,18 +706,55 @@ bool PADnoteParameters::export2wav(std::string basefilename)
             continue;
         char tmpstr[20];
         snprintf(tmpstr, 20, "_%02d", k + 1);
-        std::string filename = basefilename + std::string(tmpstr) + ".wav";
-        WavFile     wav(filename, synth->samplerate, 1);
-        if(wav.good())
+        string filename = basefilename + string(tmpstr) + EXTEN::MSwave;
+        int nsmps = sample[k].size;
+        unsigned int block;
+        unsigned short int sBlock;
+        unsigned int buffSize = 44 + sizeof(short int) * nsmps; // total size
+        char *buffer = (char*) malloc (buffSize);
+        strcpy(buffer, type.c_str());
+        block = nsmps * 4 + 36; // 2 channel shorts + part header
+        buffer[4] = block & 0xff;
+        buffer[5] = (block >> 8) & 0xff;
+        buffer[6] = (block >> 16) & 0xff;
+        buffer[7] = (block >> 24) & 0xff;
+        string temp = "WAVEfmt ";
+        strcpy(buffer + 8, temp.c_str());
+        block = 16; // subchunk size
+        memcpy(buffer + 16, &block, 4);
+        sBlock = 1; // AudioFormat uncompressed
+        memcpy(buffer + 20, &sBlock, 2);
+        sBlock = 1; // NumChannels mono
+        memcpy(buffer + 22, &sBlock, 2);
+        block = synth->samplerate;
+        memcpy(buffer + 24, &block, 4);
+        block = synth->samplerate * 2; // ByteRate
+                // (SampleRate * NumChannels * BitsPerSample) / 8
+        memcpy(buffer + 28, &block, 4);
+        sBlock = 2; // BlockAlign
+                // (bitsPerSample * channels) / 8
+        memcpy(buffer + 32, &sBlock, 2);
+        sBlock = 16; // BitsPerSample
+        memcpy(buffer + 34, &sBlock, 2);
+        temp = "data";
+        strcpy(buffer + 36, temp.c_str());
+        block = nsmps * 2; // data size
+        memcpy(buffer + 40, &block, 4);
+        for(int i = 0; i < nsmps; ++i)
         {
-            int nsmps = sample[k].size;
-            short int *smps = new short int[nsmps];
-            for(int i = 0; i < nsmps; ++i)
-                smps[i] = (short int)(sample[k].smp[i] * 32767.0f);
-            wav.writeMonoSamples(nsmps, smps);
+            sBlock = (sample[k].smp[i] * 32767.0f);
+            buffer [44 + i * 2] = sBlock & 0xff;
+            buffer [45 + i * 2] = (sBlock >> 8) & 0xff;
         }
-        else
-            isOK = false;
+        /*
+         * The file manager can return a negative number on error,
+         * so the comparison in the line below must be as integers.
+         * This is safe here as the maximum possible buffer size
+         * is well below the size of an integer.
+         */
+        isOK = (saveData(buffer, buffSize, filename) == int(buffSize));
+        free (buffer);
+
     }
     return isOK;
 }
@@ -762,7 +785,7 @@ void PADnoteParameters::add2XML(XMLwrapper *xml)
     xml->endbranch();
 
     xml->beginbranch("OSCIL");
-        oscilgen->add2XML(xml);
+        POscil->add2XML(xml);
     xml->endbranch();
 
     xml->beginbranch("RESONANCE");
@@ -864,7 +887,7 @@ void PADnoteParameters::getfromXML(XMLwrapper *xml)
 
     if (xml->enterbranch("OSCIL"))
     {
-        oscilgen->getfromXML(xml);
+        POscil->getfromXML(xml);
         xml->exitbranch();
     }
 
@@ -960,12 +983,11 @@ void PADnoteParameters::getfromXML(XMLwrapper *xml)
 
 float PADnoteParameters::getLimits(CommandBlock *getData)
 {
-    float value = getData->data.value;
-    unsigned char type = getData->data.type;
-    int request = type & TOPLEVEL::type::Default;
+    float value = getData->data.value.F;
+    int request = int(getData->data.type & TOPLEVEL::type::Default);
     int control = getData->data.control;
 
-    type &= (TOPLEVEL::source::MIDI || TOPLEVEL::source::CLI || TOPLEVEL::source::GUI); // source bits only
+    unsigned char type = 0;
 
     // padnote defaults
     int min = 0;
@@ -1035,8 +1057,8 @@ float PADnoteParameters::getLimits(CommandBlock *getData)
             break;
 
         case PADSYNTH::control::detuneType:
-            def = 0;
-            max = 3;
+            def = 1;
+            max = 4;
             break;
 
         case PADSYNTH::control::coarseDetune:
@@ -1219,13 +1241,3 @@ float PADnoteParameters::getLimits(CommandBlock *getData)
     }
     return value;
 }
-
-void PADnoteParameters::postrender(void)
-{
-    // loop over our gathered dirty flags and unset them for the next period
-      AmpLfo->updated
-    = FilterLfo->updated
-    = FreqLfo->updated
-    = false;
-
-}
