@@ -5,6 +5,7 @@
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
     Copyright 2017-2018, Will Godfrey
+    Copyright 2020 Kristian Amlie
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -21,20 +22,18 @@
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
     This file is a derivative of a ZynAddSubFX original.
-    Modified March 2018
+
 */
 
 #ifndef PAD_NOTE_PARAMETERS_H
 #define PAD_NOTE_PARAMETERS_H
 
-using namespace std;
-
 #include "Params/Presets.h"
-#include "Misc/MiscFuncs.h"
 
 class XMLwrapper;
 class FFTwrapper;
 class OscilGen;
+class OscilParameters;
 class Resonance;
 class EnvelopeParams;
 class LFOParams;
@@ -55,7 +54,6 @@ class PADnoteParameters : public Presets
         void add2XML(XMLwrapper *xml);
         void getfromXML(XMLwrapper *xml);
         float getLimits(CommandBlock *getData);
-        void postrender(void);
 
         //returns a value between 0.0-1.0 that represents the estimation
         // perceived bandwidth
@@ -157,6 +155,7 @@ class PADnoteParameters : public Presets
         void applyparameters(void);
         bool export2wav(std::string basefilename);
 
+        OscilParameters *POscil;
         OscilGen *oscilgen;
         Resonance *resonance;
 
