@@ -5,6 +5,7 @@
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009, Alan Calvert
     Copyright 2017-2018, Will Godfrey
+    Copyright 2020 Kristian Amlie
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -21,7 +22,7 @@
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
     This file is derivative of ZynAddSubFX original code.
-    Modified March 2018
+
 */
 
 #ifndef SUB_NOTE_PARAMETERS_H
@@ -46,7 +47,6 @@ class SUBnoteParameters : public Presets
         void getfromXML(XMLwrapper *xml);
         float getLimits(CommandBlock *getData);
         void updateFrequencyMultipliers(void);
-        void postrender(void);
 
         // Amplitude Parametrers
         bool Pstereo; // true = stereo, false = mono
