@@ -4,7 +4,8 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2014-2017, Will Godfrey & others
+    Copyright 2014-2019, Will Godfrey & others
+    Copyright 2020 Kristian Amlie
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -21,15 +22,15 @@
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
     This file is a derivative of a ZynAddSubFX original
-    Modified September 2017
+
 */
 
 #ifndef AD_NOTE_H
 #define AD_NOTE_H
 
-#include "Misc/SynthHelper.h"
+#include "Params/ADnoteParameters.h"
 #include "Synth/LegatoTypes.h"
-#include "Misc/Float2Int.h"
+#include "Misc/RandomGen.h"
 
 class ADnoteParameters;
 class Controller;
@@ -45,13 +46,17 @@ class Filter;
 
 class SynthEngine;
 
-class ADnote : private SynthHelper, private Float2Int
+class ADnote
 {
     public:
         ADnote(ADnoteParameters *adpars_, Controller *ctl_, float freq_, float velocity_,
                int portamento_, int midinote_, bool besilent, SynthEngine *_synth);
+        ADnote(ADnote *parent, float freq_, int subVoiceNumber_, float *parentFMmod_,
+               bool forFM_);
         ~ADnote();
 
+        void construct();
+
         int noteout(float *outl, float *outr);
         void releasekey();
         int finished() const;
@@ -61,26 +66,44 @@ class ADnote : private SynthHelper, private Float2Int
 
     private:
 
-        void setfreq(int nvoice, float in_freq);
-        void setfreqFM(int nvoice, float in_freq);
+        void setfreq(int nvoice, float in_freq, float pitchdetune);
+        void setfreqFM(int nvoice, float in_freq, float pitchdetune);
+        void setPitchDetuneFromParent(float pitch)
+        {
+            detuneFromParent = pitch;
+        }
+        void setUnisonDetuneFromParent(float factor)
+        {
+            unisonDetuneFactorFromParent = factor;
+        }
         void computeUnisonFreqRap(int nvoice);
-        void computeCurrentParameters(void);
+        void computeNoteParameters(void);
+        void computeWorkingParameters(void);
         void initParameters(void);
+        void initSubVoices(void);
         void killVoice(int nvoice);
         void killNote(void);
         float getVoiceBaseFreq(int nvoice);
         float getFMVoiceBaseFreq(int nvoice);
         void computeVoiceOscillatorLinearInterpolation(int nvoice);
-        void computeVoiceOscillatorCubicInterpolation(int nvoice);
-        void computeVoiceOscillatorMorph(int nvoice);
-        void computeVoiceOscillatorRingModulation(int nvoice);
-        void computeVoiceOscillatorFrequencyModulation(int nvoice, int FMmode);
+        void applyVoiceOscillatorMorph(int nvoice);
+        void applyVoiceOscillatorRingModulation(int nvoice);
+        void computeVoiceModulator(int nvoice, int FMmode);
+        void computeVoiceModulatorLinearInterpolation(int nvoice);
+        void normalizeVoiceModulatorFrequencyModulation(int nvoice, int FMmode);
+        void computeVoiceModulatorFrequencyModulation(int nvoice, int FMmode);
+        void computeVoiceModulatorForFMFrequencyModulation(int nvoice);
+        void computeVoiceOscillatorFrequencyModulation(int nvoice);
+        void computeVoiceOscillatorForFMFrequencyModulation(int nvoice);
             // FMmode = 0 for phase modulation, 1 for Frequency modulation
         //  void ComputeVoiceOscillatorFrequencyModulation(int nvoice);
         void computeVoiceOscillatorPitchModulation(int nvoice);
 
         void computeVoiceNoise(int nvoice);
         void ComputeVoicePinkNoise(int nvoice);
+        void ComputeVoiceSpotNoise(int nvoice);
+
+        void computeVoiceOscillator(int nvoice);
 
         void fadein(float *smps);
 
@@ -129,10 +152,11 @@ class ADnote : private SynthHelper, private Float2Int
         // Voice parameters
         struct ADnoteVoice {
             bool Enabled;
+            int Voice;        // Voice I use as source.
             int noisetype;    // (sound/noise)
             int filterbypass;
             int DelayTicks;
-            float *OscilSmp;  // Waveform of the Voice
+            float *OscilSmp;  // Waveform of the Voice. Shared with sub voices.
             int phase_offset; // PWM emulation
 
             // Frequency parameters
@@ -176,8 +200,10 @@ class ADnote : private SynthHelper, private Float2Int
             unsigned char FMFreqFixed;
             int    FMVoice;
             float *VoiceOut; // Voice Output used by other voices if use this as modullator
-            float *FMSmp;    // Wave of the Voice
+            float *FMSmp;    // Wave of the Voice. Shared by sub voices.
+            int    FMphase_offset;
             float  FMVolume;
+            bool FMDetuneFromBaseOsc;  // Whether we inherit the base oscillator's detuning
             float  FMDetune; // in cents
             Envelope *FMFreqEnvelope;
             Envelope *FMAmpEnvelope;
@@ -186,12 +212,18 @@ class ADnote : private SynthHelper, private Float2Int
         // Internal values of the note and of the voices
         float time; // time from the start of the note
 
+        RandomGen paramRNG; // A preseeded random number generator, reseeded
+                            // with a known seed every time parameters are
+                            // updated. This allows parameters to be changed
+                            // smoothly. New notes will get a new seed.
+        uint32_t paramSeed; // The seed for paramRNG.
+
         //pinking filter (Paul Kellet)
         float pinking[NUM_VOICES][14];
 
         int unison_size[NUM_VOICES]; // the size of unison for a single voice
 
-        float unison_stereo_spread[NUM_VOICES]; // stereo spread of subvoices (0.0=mono,1.0=max)
+        float unison_stereo_spread[NUM_VOICES]; // stereo spread of unison subvoices (0.0=mono,1.0=max)
 
         float *oscposlo[NUM_VOICES], *oscfreqlo[NUM_VOICES]; // fractional part (skip)
 
@@ -203,7 +235,11 @@ class ADnote : private SynthHelper, private Float2Int
 
         float *unison_freq_rap[NUM_VOICES]; // how the unison subvoice's frequency is changed (1.0 for no change)
 
-        bool *unison_invert_phase[NUM_VOICES]; // which subvoice has phase inverted
+        // These are set by parent voices.
+        float detuneFromParent;             // How much the voice should be detuned.
+        float unisonDetuneFactorFromParent; // How much the voice should be detuned from unison.
+
+        bool *unison_invert_phase[NUM_VOICES]; // which unison subvoice has phase inverted
 
         struct { // unison vibratto
             float  amplitude; // amplitude which be added to unison_freq_rap
@@ -222,13 +258,17 @@ class ADnote : private SynthHelper, private Float2Int
 
         float *FMoldsmp[NUM_VOICES]; // used by Frequency Modulation (for integration)
 
-        float *tmpwavel; // temporary buffers
-        float *tmpwaver;
+        float *FMFMoldsmpModded[NUM_VOICES]; // use when rendering FM modulator with parent FM
+        float *FMFMoldsmpOrig[NUM_VOICES];
+        float *oscFMoldsmpModded[NUM_VOICES]; // use when rendering oscillator for FM with parent FM
+        float *oscFMoldsmpOrig[NUM_VOICES];
+        bool forFM; // Whether this voice will be used for FM modulation.
+
         float **tmpwave_unison;
         int max_unison;
 
-        float *bypassl; // Filter bypass samples
-        float *bypassr;
+        float **tmpmod_unison;
+        bool freqbasedmod[NUM_VOICES];
 
         float globaloldamplitude; // interpolate the amplitudes
         float globalnewamplitude;
@@ -264,6 +304,19 @@ class ADnote : private SynthHelper, private Float2Int
         float pangainL;
         float pangainR;
 
+        ADnote **subVoice[NUM_VOICES];
+        ADnote **subFMVoice[NUM_VOICES];
+
+        int subVoiceNumber;
+        // For sub voices: The original, "topmost" voice that triggered this
+        // one.
+        ADnote *origVoice;
+        // For sub voices: Pointer to the closest parent that has
+        // phase/frequency modulation.
+        float *parentFMmod;
+
+        Presets::PresetsUpdate paramsUpdate;
+
         SynthEngine *synth;
 };
 
