@@ -4,7 +4,7 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2017-2018 Will Godfrey & others.
+    Copyright 2017-2019 Will Godfrey & others.
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -22,37 +22,42 @@
 
     This file is derivative of original ZynAddSubFX code.
 
-    Modified December 2018
+    Modified May 2019
 */
 
 #ifndef OSCIL_GEN_H
 #define OSCIL_GEN_H
 
+#include <sys/types.h>
 #include <limits.h>
 
 #include "Misc/RandomGen.h"
 #include "Misc/WaveShapeSamples.h"
 #include "Misc/XMLwrapper.h"
-#include "DSP/FFTwrapper.h"
-#include "Params/Presets.h"
+#include "Params/OscilParameters.h"
 #include "Synth/Resonance.h"
 
 class SynthEngine;
 
-class OscilGen : public Presets, private WaveShapeSamples
+class OscilGen : private WaveShapeSamples
 {
     public:
-        OscilGen(FFTwrapper *fft_,Resonance *res_, SynthEngine *_synth);
+        OscilGen(FFTwrapper *fft_,Resonance *res_, SynthEngine *_synth, OscilParameters *params_);
         ~OscilGen();
 
+        void changeParams(OscilParameters *params_);
+
         void prepare();
 
-        int get(float *smps, float freqHz);
+        void get(float *smps, float freqHz);
         // returns where should I start getting samples, used in block type randomness
 
-        int get(float *smps, float freqHz, int resonance);
+        void get(float *smps, float freqHz, int resonance);
         // if freqHz is smaller than 0, return the "un-randomized" sample for UI
 
+        // Get just the phase of the oscillator.
+        int getPhase();
+
         void getbasefunction(float *smps);
 
         // called by UI
@@ -60,67 +65,19 @@ class OscilGen : public Presets, private WaveShapeSamples
         void getcurrentbasefunction(float *smps);
         void useasbase(void);
 
-        void add2XML(XMLwrapper *xml);
+        void genDefaults(void);
         void defaults(void);
-        void getfromXML(XMLwrapper *xml);
-        float getLimits(CommandBlock *getData);
         void convert2sine();
 
         // Make a new random seed for Amplitude Randomness -
         //   should be called every noteon event
         void newrandseed() { randseed = prng.randomINT() + INT_MAX/2; }
 
-        // Parameters
-
-        /**
-         * The hmag and hphase starts counting from 0, so the first harmonic(1) has the index 0,
-         * 2-nd harmonic has index 1, ..the 128 harminic has index 127
-         */
-        unsigned char Phmag[MAX_AD_HARMONICS], Phphase[MAX_AD_HARMONICS];
-        // the MIDI parameters for mag. and phases
-
-        unsigned char Phmagtype; // 0 - Linear, 1 - dB scale (-40), 2 - dB scale (-60)
-                                 // 3 - dB scale (-80), 4 - dB scale (-100)
-        unsigned char Pcurrentbasefunc; // The base function used - 0=sin, 1=...
-        unsigned char Pbasefuncpar; // the parameter of the base function
-
-        unsigned char Pbasefuncmodulation; // what modulation is applied to the
-                                           // basefunc
-        unsigned char Pbasefuncmodulationpar1,
-                      Pbasefuncmodulationpar2,
-                      Pbasefuncmodulationpar3; // the parameter of the base
-                                               // function modulation
-
-        unsigned char Prand; // 64 = no randomness
-                             // 63..0 - block type randomness - 0 is maximum
-                             // 65..127 - each harmonic randomness - 127 is maximum
-        unsigned char Pwaveshaping, Pwaveshapingfunction;
-        unsigned char Pfiltertype, Pfilterpar1, Pfilterpar2;
-        unsigned char Pfilterbeforews;
-        unsigned char Psatype, Psapar; // spectrum adjust
-
-        unsigned char Pamprandpower, Pamprandtype; // amplitude randomness
-        int Pharmonicshift; // how the harmonics are shifted
-        int Pharmonicshiftfirst; // if the harmonic shift is done before
-                                 // waveshaping and filter
-
-        unsigned char Padaptiveharmonics; // the adaptive harmonics status
-                                          // (off=0,on=1,etc..)
-        unsigned char Padaptiveharmonicsbasefreq; // the base frequency of the
-                                                  // adaptive harmonic (30..3000Hz)
-        unsigned char Padaptiveharmonicspower; // the strength of the effect
-                                               // (0=off,100=full)
-        unsigned char Padaptiveharmonicspar; // the parameters in 2,3,4.. modes
-                                             // of adaptive harmonics
-
-        unsigned char Pmodulation; // what modulation is applied to the oscil
-        unsigned char Pmodulationpar1,
-                      Pmodulationpar2,
-                      Pmodulationpar3; // the parameter of the parameters
-
-        bool ADvsPAD; // if it is used by ADsynth or by PADsynth
-
     private:
+        OscilParameters *params;
+
+        SynthEngine *synth;
+
         float *tmpsmps;
         FFTFREQS outoscilFFTfreqs;
 
@@ -189,14 +146,14 @@ class OscilGen : public Presets, private WaveShapeSamples
             oldmodulationpar2,
             oldmodulationpar3;
 
-        FFTFREQS basefuncFFTfreqs; // Base Function Frequencies
         FFTFREQS oscilFFTfreqs; // Oscillator Frequencies - this is different
                                 // than the hamonics set-up by the user, it may
                                 // contain time-domain data if the antialiasing
                                 // is turned off
-        int oscilprepared; // 1 if the oscil is prepared, 0 if it is not
-                           // prepared and is need to call ::prepare() before
-                           // ::get()
+        Presets::PresetsUpdate oscilupdate;// whether the oscil is prepared, if
+                                           // not prepared we need to call
+                                           // ::prepare() before ::get()
+
 
         Resonance *res;
 
