@@ -5,6 +5,7 @@
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2010 Alan Calvert
     Copyright 2017 Will Godfrey & others
+    Copyright 2020 Kristian Amlie
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -21,13 +22,12 @@
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
     This file is a derivative of the ZynAddSubFX original
-    Modified September 2017
+
 */
 
 #ifndef PAD_NOTE_H
 #define PAD_NOTE_H
 
-#include "Misc/SynthHelper.h"
 #include "Synth/LegatoTypes.h"
 
 class PADnoteParameters;
@@ -39,7 +39,7 @@ class Controller;
 
 class SynthEngine;
 
-class PADnote : private SynthHelper
+class PADnote
 {
     public:
         PADnote(PADnoteParameters *parameters, Controller *ctl_, float freq,
@@ -57,7 +57,9 @@ class PADnote : private SynthHelper
 
     private:
         void fadein(float *smps);
+        void computeNoteParameters();
         void computecurrentparameters();
+        void setBaseFreq();
         bool finished_;
         PADnoteParameters *pars;
 
@@ -153,6 +155,8 @@ class PADnote : private SynthHelper
             } param;
         } Legato;
 
+        Presets::PresetsUpdate padSynthUpdate;
+
         SynthEngine *synth;
 };
 
