@@ -5,7 +5,8 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2009 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2014-2018, Will Godfrey & others
+    Copyright 2014-2019, Will Godfrey & others
+    Copyright 2020 Kristian Amlie
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -22,7 +23,7 @@
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
     This file is derivative of ZynAddSubFX original code
-    Modified October 2018
+
 */
 
 #include <cmath>
@@ -34,8 +35,15 @@
 #include "Synth/Envelope.h"
 #include "DSP/Filter.h"
 #include "Misc/SynthEngine.h"
+#include "Misc/SynthHelper.h"
 #include "Synth/SUBnote.h"
 
+using synth::velF;
+using synth::getDetune;
+using synth::interpolateAmplitude;
+using synth::aboveAmplitudeThreshold;
+
+
 SUBnote::SUBnote(SUBnoteParameters *parameters, Controller *ctl_, float freq,
                  float velocity, int portamento_, int midinote, bool besilent, SynthEngine *_synth) :
     pars(parameters),
@@ -48,6 +56,7 @@ SUBnote::SUBnote(SUBnoteParameters *parameters, Controller *ctl_, float freq,
     log_0_001(logf(0.001f)),
     log_0_0001(logf(0.0001f)),
     log_0_00001(logf(0.00001f)),
+    subNoteChange(parameters),
     synth(_synth),
     filterStep(0)
 {
@@ -55,8 +64,7 @@ SUBnote::SUBnote(SUBnoteParameters *parameters, Controller *ctl_, float freq,
 
     // Initialise some legato-specific vars
     Legato.msg = LM_Norm;
-    FR2Z2I(synth->samplerate_f * 0.005f, Legato.fade.length); // 0.005 seems ok.
-    //Legato.fade.length = (int)truncf(synth->samplerate_f * 0.005f); // 0.005 seems ok.
+    Legato.fade.length = int(synth->samplerate_f * 0.005f); // 0.005 seems ok.
     if (Legato.fade.length < 1)
         Legato.fade.length = 1;// (if something's fishy)
     Legato.fade.step = (1.0f / Legato.fade.length);
@@ -68,89 +76,33 @@ SUBnote::SUBnote(SUBnoteParameters *parameters, Controller *ctl_, float freq,
     Legato.silent = besilent;
 
     NoteEnabled = true;
-    volume = powf(0.1f, 3.0f * (1.0f - pars->PVolume / 96.0f)); // -60 dB .. 0 dB
-    volume *= velF(velocity, pars->PAmpVelocityScaleFunction);
-    if (pars->randomPan())
-    {
-        float t = synth->numRandom();
-        randpanL = cosf(t * HALFPI);
-        randpanR = cosf((1.0f - t) * HALFPI);
-    }
-    else
-        randpanL = randpanR = 0.7f;
+
     numstages = pars->Pnumstages;
     stereo = pars->Pstereo;
     start = pars->Pstart;
     firsttick = 1;
 
-    if (pars->Pfixedfreq == 0)
-        basefreq = freq;
-    else
-    {
-        basefreq = 440.0f;
-        int fixedfreqET = pars->PfixedfreqET;
-        if (fixedfreqET)
-        {   // if the frequency varies according the keyboard note
-            float tmp =
-                (midinote - 69.0f) / 12.0f * powf(2.0f, (((fixedfreqET - 1) / 63.0f) - 1.0f));
-            if (fixedfreqET <= 64)
-                basefreq *= powf(2.0f, tmp);
-            else
-                basefreq *= powf(3.0f, tmp);
-        }
-    }
+    float t = synth->numRandom();
+    randpanL = cosf(t * HALFPI);
+    randpanR = cosf((1.0f - t) * HALFPI);
 
-    int BendAdj = pars->PBendAdjust - 64;
-    if (BendAdj % 24 == 0)
-        BendAdjust = BendAdj / 24;
-    else
-        BendAdjust = BendAdj / 24.0f;
-    float offset_val = (pars->POffsetHz - 64)/64.0f;
-    OffsetHz = 15.0f*(offset_val * sqrtf(fabsf(offset_val)));
-
-    float detune = getDetune(pars->PDetuneType, pars->PCoarseDetune, pars->PDetune);
-    basefreq *= powf(2.0f, detune / 1200.0f); // detune
-//    basefreq*=ctl->pitchwheel.relfreq;//pitch wheel
-
-    // global filter
-    GlobalFilterCenterPitch =
-        pars->GlobalFilter->getfreq()
-        + // center freq
-          (pars->PGlobalFilterVelocityScale / 127.0f * 6.0f)
-        * // velocity sensing
-          (velF(velocity, pars->PGlobalFilterVelocityScaleFunction) - 1);
-
-    // select only harmonics that desire to compute
     numharmonics = 0;
-    for (int n = 0; n < MAX_SUB_HARMONICS; ++n)
-    {
-        if (pars->Phmag[n] == 0)
-            continue;
-        if (n * basefreq > synth->halfsamplerate_f)
-            break; // remove the freqs above the Nyquist freq
-        pos[numharmonics++] = n;
-    }
-    firstnumharmonics = numharmonics; // (gf)Useful in legato mode.
+    lfilter = NULL;
+    rfilter = NULL;
 
-    if (numharmonics == 0)
-    {
-        NoteEnabled = false;
-        return;
-    }
-
-    lfilter = new bpfilter[numstages * numharmonics];
-    if (stereo != 0)
-        rfilter = new bpfilter[numstages * numharmonics];
-
-    initfilterbank();
+    setBaseFreq();
 
     oldpitchwheel = 0;
     oldbandwidth = 64;
+
     if (pars->Pfixedfreq == 0)
         initparameters(basefreq);
     else
         initparameters(basefreq / 440.0f * freq);
 
+    computeNoteParameters();
+    computecurrentparameters();
+
     oldamplitude = newamplitude;
     ready = 1;
 }
@@ -193,80 +145,12 @@ void SUBnote::SUBlegatonote(float freq, float velocity,
 
     portamento = portamento_;
 
-    volume = powf(0.1f, 3.0f * (1.0f - pars->PVolume / 96.0f)); // -60 dB .. 0 dB
-    volume *= velF(velocity, pars->PAmpVelocityScaleFunction);
-    if (pars->randomPan())
-    {
-        float t = synth->numRandom();
-        randpanL = cosf(t * HALFPI);
-        randpanR = cosf((1.0f - t) * HALFPI);
-    }
-    else
-        randpanL = randpanR = 0.7f;
-
-    if (pars->Pfixedfreq == 0)
-        basefreq = freq;
-    else
-    {
-        basefreq = 440.0f;
-        int fixedfreqET = pars->PfixedfreqET;
-        if (fixedfreqET != 0)
-        {   //if the frequency varies according the keyboard note
-            float tmp = (midinote - 69.0f) / 12.0f
-                              * (powf(2.0f, (fixedfreqET - 1) / 63.0f) - 1.0f);
-            if (fixedfreqET <= 64)
-                basefreq *= powf(2.0f, tmp);
-            else
-                basefreq *= powf(3.0f, tmp);
-        }
-    }
-    float detune = getDetune(pars->PDetuneType, pars->PCoarseDetune, pars->PDetune);
-    basefreq *= powf(2.0f, detune / 1200.0f); // detune
-
-    // global filter
-    GlobalFilterCenterPitch = pars->GlobalFilter->getfreq() + // center freq
-                              (pars->PGlobalFilterVelocityScale / 127.0f * 6.0f)
-                              // velocity sensing
-                              * (velF(velocity, pars->PGlobalFilterVelocityScaleFunction) - 1);
-
-
-    int legatonumharmonics = 0;
-    for (int n = 0; n < MAX_SUB_HARMONICS; ++n)
-    {
-        if (pars->Phmag[n] == 0)
-            continue;
-        if (n * basefreq > synth->samplerate_f / 2.0f)
-            break; // remove the freqs above the Nyquist freq
-        pos[legatonumharmonics++] = n;
-    }
-    if (legatonumharmonics > firstnumharmonics)
-        numharmonics = firstnumharmonics;
-    else
-        numharmonics = legatonumharmonics;
-
-    if (numharmonics == 0)
-    {
-        NoteEnabled = false;
-        return;
-    }
-
-    initfilterbank();
+    setBaseFreq();
 
     oldpitchwheel = 0;
     oldbandwidth = 64;
 
-    if (pars->Pfixedfreq == 0)
-        freq = basefreq;
-    else
-        freq *= basefreq / 440.0f;
-
-    // Altered initparameters(...) content:
-    if (pars->PGlobalFilterEnabled)
-    {
-        globalfiltercenterq = pars->GlobalFilter->getq();
-        GlobalFilterFreqTracking = pars->GlobalFilter->getfreqtracking(basefreq);
-    }
-    // end of the altered initparameters function content.
+    computeNoteParameters();
 
     oldamplitude = newamplitude;
 
@@ -300,6 +184,103 @@ void SUBnote::KillNote(void)
     }
 }
 
+int SUBnote::createNewFilters()
+{
+    bool alreadyEnabled[MAX_SUB_HARMONICS];
+    memset(alreadyEnabled, 0, sizeof(alreadyEnabled));
+    for (int p = 0; p < numharmonics; ++p)
+        alreadyEnabled[pos[p]] = true;
+
+    // select only harmonics that desire to compute
+    int origNumHarmonics = numharmonics;
+    for (int n = 0; n < MAX_SUB_HARMONICS; ++n)
+    {
+        if (pars->Phmag[n] == 0 || alreadyEnabled[n])
+            continue;
+        if (n * basefreq > synth->halfsamplerate_f)
+            break; // remove the freqs above the Nyquist freq
+        pos[numharmonics++] = n;
+        alreadyEnabled[n] = true;
+    }
+
+    if (numharmonics == origNumHarmonics)
+        return 0;
+
+    bpfilter *newFilter = new bpfilter[numstages * numharmonics];
+    if (lfilter != NULL)
+    {
+        memcpy(newFilter, lfilter, numstages * origNumHarmonics * sizeof(bpfilter));
+        delete [] lfilter;
+    }
+    lfilter = newFilter;
+    if (stereo != 0)
+    {
+        newFilter = new bpfilter[numstages * numharmonics];
+        if (rfilter != NULL)
+        {
+            memcpy(newFilter, rfilter, numstages * origNumHarmonics * sizeof(bpfilter));
+            delete [] rfilter;
+        }
+        rfilter = newFilter;
+    }
+
+    return numharmonics - origNumHarmonics;
+}
+
+void SUBnote::setBaseFreq()
+{
+    if (pars->Pfixedfreq == 0)
+        basefreq = Legato.param.freq;
+    else
+    {
+        basefreq = 440.0f;
+        int fixedfreqET = pars->PfixedfreqET;
+        if (fixedfreqET)
+        {   // if the frequency varies according the keyboard note
+            float tmp =
+                (Legato.param.midinote - 69.0f) / 12.0f * powf(2.0f, (((fixedfreqET - 1) / 63.0f) - 1.0f));
+            if (fixedfreqET <= 64)
+                basefreq *= powf(2.0f, tmp);
+            else
+                basefreq *= powf(3.0f, tmp);
+        }
+    }
+
+    float detune = getDetune(pars->PDetuneType, pars->PCoarseDetune, pars->PDetune);
+    basefreq *= powf(2.0f, detune / 1200.0f); // detune
+//    basefreq*=ctl->pitchwheel.relfreq;//pitch wheel
+}
+
+void SUBnote::computeNoteParameters()
+{
+    volume = powf(0.1f, 3.0f * (1.0f - pars->PVolume / 96.0f)); // -60 dB .. 0 dB
+    volume *= velF(Legato.param.vel, pars->PAmpVelocityScaleFunction);
+
+    int BendAdj = pars->PBendAdjust - 64;
+    if (BendAdj % 24 == 0)
+        BendAdjust = BendAdj / 24;
+    else
+        BendAdjust = BendAdj / 24.0f;
+    float offset_val = (pars->POffsetHz - 64)/64.0f;
+    OffsetHz = 15.0f*(offset_val * sqrtf(fabsf(offset_val)));
+
+    // global filter
+    GlobalFilterCenterPitch =
+        pars->GlobalFilter->getfreq()
+        + // center freq
+          (pars->PGlobalFilterVelocityScale / 127.0f * 6.0f)
+        * // velocity sensing
+          (velF(Legato.param.vel, pars->PGlobalFilterVelocityScaleFunction) - 1);
+
+    updatefilterbank();
+
+    if (pars->PGlobalFilterEnabled != 0)
+    {
+        globalfiltercenterq = pars->GlobalFilter->getq();
+        GlobalFilterFreqTracking = pars->GlobalFilter->getfreqtracking(basefreq);
+    }
+}
+
 // Compute the filters coefficients
 void SUBnote::computefiltercoefs(bpfilter &filter, float freq, float bw, float gain)
 {
@@ -326,7 +307,22 @@ void SUBnote::computefiltercoefs(bpfilter &filter, float freq, float bw, float g
 
 
 // Initialise the filters
-void SUBnote::initfilter(bpfilter &filter, float freq, float bw, float amp, float mag)
+void SUBnote::initfilters(int startIndex)
+{
+    for (int n = startIndex; n < numharmonics; ++n)
+    {
+        float hgain = getHgain(n);
+
+        for (int nph = 0; nph < numstages; ++nph)
+        {
+            initfilter(lfilter[nph + n * numstages], hgain);
+            if (stereo)
+                initfilter(rfilter[nph + n * numstages], hgain);
+        }
+    }
+}
+
+void SUBnote::initfilter(bpfilter &filter, float mag)
 {
     filter.xn1 = 0.0f;
     filter.xn2 = 0.0f;
@@ -343,21 +339,16 @@ void SUBnote::initfilter(bpfilter &filter, float freq, float bw, float amp, floa
         if (start == 1)
             a *= synth->numRandom();
         filter.yn1 = a * cosf(p);
-        filter.yn2 = a * cosf(p + freq * TWOPI / synth->samplerate_f);
+        filter.yn2 = a * cosf(p + filter.freq * TWOPI / synth->samplerate_f);
 
         // correct the error of computation the start amplitude
         // at very high frequencies
-        if (freq > synth->samplerate_f * 0.96f)
+        if (filter.freq > synth->samplerate_f * 0.96f)
         {
             filter.yn1 = 0.0f;
             filter.yn2 = 0.0f;
         }
     }
-
-    filter.amp = amp;
-    filter.freq = freq;
-    filter.bw = bw;
-    computefiltercoefs(filter, freq, bw, 1.0f);
 }
 
 
@@ -474,14 +465,11 @@ void SUBnote::initparameters(float freq)
         BandWidthEnvelope = NULL;
     if (pars->PGlobalFilterEnabled != 0)
     {
-        globalfiltercenterq = pars->GlobalFilter->getq();
         GlobalFilterL = new Filter(pars->GlobalFilter, synth);
         if (stereo != 0)
             GlobalFilterR = new Filter(pars->GlobalFilter, synth);
         GlobalFilterEnvelope = new Envelope(pars->GlobalFilterEnvelope, freq, synth);
-        GlobalFilterFreqTracking = pars->GlobalFilter->getfreqtracking(basefreq);
     }
-    computecurrentparameters();
 }
 //end of port
 
@@ -504,6 +492,68 @@ float SUBnote::computerolloff(float freq)
     return (1.0f - cosf(M_PI * (freq - upper_limit) / upper_width)) / 2.0f;
 }
 
+void SUBnote::computeallfiltercoefs()
+{
+    float envfreq = 1.0f;
+    float envbw = 1.0f;
+    float gain = 1.0f;
+
+    if (FreqEnvelope != NULL)
+    {
+        envfreq = FreqEnvelope->envout() / 1200;
+        envfreq = powf(2.0f, envfreq);
+    }
+
+    envfreq *= powf(ctl->pitchwheel.relfreq, BendAdjust); // pitch wheel
+
+    if (portamento != 0)
+    {   // portamento is used
+        envfreq *= ctl->portamento.freqrap;
+        if (ctl->portamento.used == 0)
+        {   // the portamento has finished
+            portamento = 0; // this note is no longer "portamented"
+        }
+    }
+
+    if (BandWidthEnvelope != NULL)
+    {
+        envbw = BandWidthEnvelope->envout();
+        envbw = powf(2.0f, envbw);
+    }
+    envbw *= ctl->bandwidth.relbw; // bandwidth controller
+
+    float tmpgain = 1.0f / sqrtf(envbw * envfreq);
+
+    for (int n = 0; n < numharmonics; ++n)
+    {
+        for (int nph = 0; nph < numstages; ++nph)
+        {
+            if (nph == 0)
+                gain = tmpgain;
+            else
+                gain = 1.0f;
+            computefiltercoefs( lfilter[nph + n * numstages],
+                                lfilter[nph + n *numstages].freq * envfreq,
+                                lfilter[nph + n * numstages].bw * envbw, gain);
+        }
+    }
+    if (stereo)
+        for (int n = 0; n < numharmonics; ++n)
+        {
+            for (int nph = 0; nph < numstages; ++nph)
+            {
+                if (nph == 0)
+                    gain = tmpgain;
+                else
+                    gain = 1.0f;
+                computefiltercoefs( rfilter[nph + n * numstages],
+                                    rfilter[nph + n * numstages].freq * envfreq,
+                                    rfilter[nph + n * numstages].bw * envbw, gain);
+            }
+        }
+    oldbandwidth = ctl->bandwidth.data;
+    oldpitchwheel = ctl->pitchwheel.data;
+}
 
 // Compute Parameters of SUBnote for each tick
 void SUBnote::computecurrentparameters(void)
@@ -527,67 +577,7 @@ void SUBnote::computecurrentparameters(void)
         || oldpitchwheel != ctl->pitchwheel.data
         || oldbandwidth != ctl->bandwidth.data
         || portamento != 0)
-    {
-        float envfreq = 1.0f;
-        float envbw = 1.0f;
-        float gain = 1.0f;
-
-        if (FreqEnvelope != NULL)
-        {
-            envfreq = FreqEnvelope->envout() / 1200;
-            envfreq = powf(2.0f, envfreq);
-        }
-
-        envfreq *= powf(ctl->pitchwheel.relfreq, BendAdjust); // pitch wheel
-
-        if (portamento != 0)
-        {   // portamento is used
-            envfreq *= ctl->portamento.freqrap;
-            if (ctl->portamento.used == 0)
-            {   // the portamento has finished
-                portamento = 0; // this note is no longer "portamented"
-            }
-        }
-
-        if (BandWidthEnvelope != NULL)
-        {
-            envbw = BandWidthEnvelope->envout();
-            envbw = powf(2.0f, envbw);
-        }
-        envbw *= ctl->bandwidth.relbw; // bandwidth controller
-
-        float tmpgain = 1.0f / sqrtf(envbw * envfreq);
-
-        for (int n = 0; n < numharmonics; ++n)
-        {
-            for (int nph = 0; nph < numstages; ++nph)
-            {
-                if (nph == 0)
-                    gain = tmpgain;
-                else
-                    gain = 1.0f;
-                computefiltercoefs( lfilter[nph + n * numstages],
-                                    lfilter[nph + n *numstages].freq * envfreq,
-                                    lfilter[nph + n * numstages].bw * envbw, gain);
-            }
-        }
-        if (stereo)
-            for (int n = 0; n < numharmonics; ++n)
-            {
-                for (int nph = 0; nph < numstages; ++nph)
-                {
-                    if (nph == 0)
-                        gain = tmpgain;
-                    else
-                        gain = 1.0f;
-                    computefiltercoefs( rfilter[nph + n * numstages],
-                                        rfilter[nph + n * numstages].freq * envfreq,
-                                        rfilter[nph + n * numstages].bw * envbw, gain);
-                }
-            }
-        oldbandwidth = ctl->bandwidth.data;
-        oldpitchwheel = ctl->pitchwheel.data;
-    }
+        computeallfiltercoefs();
     newamplitude = volume * AmpEnvelope->envout_dB() * 2.0f;
 
     // Filter
@@ -614,6 +604,12 @@ int SUBnote::noteout(float *outl, float *outr)
     if (!NoteEnabled)
         return 0;
 
+    if (subNoteChange.checkUpdated())
+    {
+        setBaseFreq();
+        computeNoteParameters();
+    }
+
     // left channel
     for (int i = 0; i < synth->sent_buffersize; ++i)
         tmprnd[i] = synth->numRandom() * 2.0f - 1.0f;
@@ -741,7 +737,6 @@ int SUBnote::noteout(float *outl, float *outr)
                     break;
                 }
                 Legato.fade.m += Legato.fade.step;
-                Legato.fade.m = Legato.fade.m;
                 outl[i] *= Legato.fade.m;
                 outr[i] *= Legato.fade.m;
             }
@@ -773,7 +768,6 @@ int SUBnote::noteout(float *outl, float *outr)
                     break;
                 }
                 Legato.fade.m -= Legato.fade.step;
-                Legato.fade.m = Legato.fade.m;
                 outl[i] *= Legato.fade.m;
                 outr[i] *= Legato.fade.m;
             }
@@ -810,8 +804,40 @@ void SUBnote::releasekey(void)
         GlobalFilterEnvelope->releasekey();
 }
 
-void SUBnote::initfilterbank(void)
+float SUBnote::getHgain(int harmonic)
 {
+    float hmagnew = 1.0f - pars->Phmag[pos[harmonic]] / 127.0f;
+    float hgain;
+
+    switch (pars->Phmagtype)
+    {
+        case 1:
+            hgain = expf(hmagnew * log_0_01);
+            break;
+
+        case 2:
+            hgain = expf(hmagnew * log_0_001);
+            break;
+
+        case 3:
+            hgain = expf(hmagnew * log_0_0001);
+            break;
+
+        case 4:
+            hgain = expf(hmagnew * log_0_00001);
+            break;
+
+        default:
+            hgain = 1.0f - hmagnew;
+    }
+
+    return hgain;
+}
+
+void SUBnote::updatefilterbank(void)
+{
+    int createdFilters = createNewFilters();
+
     // moved from noteon
     // how much the amplitude is normalised (because the harmonics)
     float reduceamp = 0.0;
@@ -837,30 +863,8 @@ void SUBnote::initfilterbank(void)
         // try to keep same amplitude on all freqs and bw. (empirically)
         float gain = sqrtf(1500.0f / (bw * freq));
 
-        float hmagnew = 1.0f - pars->Phmag[pos[n]] / 127.0f;
-        float hgain;
+        float hgain = getHgain(n);
 
-        switch (pars->Phmagtype)
-        {
-            case 1:
-                hgain = expf(hmagnew * log_0_01);
-                break;
-
-            case 2:
-                hgain = expf(hmagnew * log_0_001);
-                break;
-
-            case 3:
-                hgain = expf(hmagnew * log_0_0001);
-                break;
-
-            case 4:
-                hgain = expf(hmagnew * log_0_00001);
-                break;
-
-            default:
-                hgain = 1.0f - hmagnew;
-        }
         gain *= hgain;
         reduceamp += hgain;
 
@@ -869,12 +873,23 @@ void SUBnote::initfilterbank(void)
             float amp = 1.0f;
             if (nph == 0)
                 amp = gain;
-            initfilter(lfilter[nph + n * numstages], freq + OffsetHz, bw, amp, hgain);
+            bpfilter *filter = &lfilter[nph + n * numstages];
+            filter->amp = amp;
+            filter->freq = freq + OffsetHz;
+            filter->bw = bw;
             if (stereo)
-                initfilter(rfilter[nph + n * numstages], freq + OffsetHz, bw, amp, hgain);
+            {
+                filter = &rfilter[nph + n * numstages];
+                filter->amp = amp;
+                filter->freq = freq + OffsetHz;
+                filter->bw = bw;
+            }
         }
     }
 
+    initfilters(numharmonics - createdFilters);
+    computeallfiltercoefs();
+
     if (reduceamp < 0.001f)
         reduceamp = 1.0f;
     volume /= reduceamp;
