@@ -27,7 +27,6 @@
 
     This file is a derivative of the ZynAddSubFX original.
 
-    Modified March 2018
 */
 
 #include "DynamicTooltip.h"
@@ -62,6 +61,7 @@ DynTooltip::DynTooltip():Fl_Menu_Window(1,1)
 
     valueType = VC_plainValue;
     graphicsType = VC_plainValue;
+    onlyValue = false;
 
     positioned = false;
     yoffs = 20;
