@@ -24,10 +24,10 @@
 
     This file is a derivative of the ZynAddSubFX original.
 
-    Modified August 2018
 */
 
 #include "WidgetPDial.h"
+#include "Misc/NumericFuncs.h"
 
 #include <FL/fl_draw.H>
 #include <FL/Fl_Tooltip.H>
@@ -36,6 +36,8 @@
 #include <cairo.h>
 #include <cairo-xlib.h>
 
+using func::limit;
+
 WidgetPDial::WidgetPDial(int x,int y, int w, int h, const char *label) : Fl_Dial(x,y,w,h,label)
 {
     Fl_Group *save = Fl_Group::current();
