@@ -2,7 +2,7 @@
     main.cpp
 
     Copyright 2009-2011, Alan Calvert
-    Copyright 2014-2018, Will Godfrey & others
+    Copyright 2014-2020, Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can
     redistribute it and/or modify it under the terms of the GNU General
@@ -17,7 +17,6 @@
     You should have received a copy of the GNU General Public License
     along with yoshimi.  If not, see <http://www.gnu.org/licenses/>.
 
-    Modified October 2018
 */
 
 // approx timeout in seconds.
@@ -25,56 +24,91 @@
 
 #include <sys/mman.h>
 #include <iostream>
+#include <string>
 #include <stdio.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <termios.h>
 #include <time.h>
 
-
-using namespace std;
-
-#include "Misc/Config.h"
-#include "Misc/Splash.h"
-#include "Misc/SynthEngine.h"
-#include "MusicIO/MusicClient.h"
-#include "MasterUI.h"
-#include "UI/MiscGui.h"
 #include <map>
 #include <list>
 #include <pthread.h>
+#include <thread>
 #include <semaphore.h>
 #include <cstdio>
 #include <unistd.h>
-
-#include <FL/Fl.H>
-#include <FL/Fl_Window.H>
-#include <FL/Fl_PNG_Image.H>
-
 #include <readline/readline.h>
 #include <readline/history.h>
-#include <Interface/CmdInterface.h>
 
-CmdInterface commandInt;
+#include "Misc/Config.h"
+#include "Misc/SynthEngine.h"
+#include "MusicIO/MusicClient.h"
+#include "CLI/CmdInterface.h"
+#include "Interface/InterChange.h"
+#include "Misc/FileMgrFuncs.h"
+#include "Misc/FormatFuncs.h"
+
+#ifdef GUI_FLTK
+    #include "MasterUI.h"
+    #include "UI/MiscGui.h"
+    #include <FL/Fl.H>
+    #include <FL/Fl_Window.H>
+    #include <FL/Fl_PNG_Image.H>
+    #include "Misc/Splash.h"
+#endif
 
 extern map<SynthEngine *, MusicClient *> synthInstances;
 extern SynthEngine *firstSynth;
-extern int startInstance;
+
 
 void mainRegisterAudioPort(SynthEngine *s, int portnum);
-int mainCreateNewInstance(unsigned int forceId, bool loadState);
-
+int mainCreateNewInstance(unsigned int forceId);
 Config *firstRuntime = NULL;
 static int globalArgc = 0;
 static char **globalArgv = NULL;
+bool isSingleMaster = false;
 bool bShowGui = true;
 bool bShowCmdLine = true;
+bool showSplash = false;
 bool splashSet = true;
+bool configuring = false;
+#ifdef GUI_FLTK
 time_t old_father_time, here_and_now;
+#endif
 
 //Andrew Deryabin: signal handling moved to main from Config Runtime
 //It's only suitable for single instance app support
 static struct sigaction yoshimiSigAction;
 
+
+void newBlock()
+{
+    for (int i = 1; i < 32; ++i)
+    {
+        if ((firstRuntime->activeInstance >> i) & 1)
+        {
+            while(configuring)
+                usleep(1000);
+            // in case there is still an instance starting from elsewhere
+            configuring = true;
+            mainCreateNewInstance(i);
+            configuring = false;
+        }
+    }
+}
+
+
+void newInstance()
+{
+    while (configuring)
+        usleep(1000);
+    // in case there is still an instance starting from elsewhere
+    configuring = true;
+    startInstance = 0x81;
+}
+
+
 void yoshimiSigHandler(int sig)
 {
     switch (sig)
@@ -90,20 +124,55 @@ void yoshimiSigHandler(int sig)
             firstRuntime->setLadi1Active();
             sigaction(SIGUSR1, &yoshimiSigAction, NULL);
             break;
-
+        case SIGUSR2: // start next instance
+            if(isSingleMaster)
+                newInstance();
+            sigaction(SIGUSR2, &yoshimiSigAction, NULL);
+            break;
         default:
             break;
     }
 }
 
+#ifdef GUI_FLTK
+void do_start(void)
+{
+    std::string startup = YOSHIMI_VERSION;
+    startup = "Yoshimi V " + startup + " is starting";
+    int w = 300;
+    int h = 36;
+    int sx = (Fl::w() - w) / 2;
+    int sy = (Fl::h() - h) / 2;
+    Fl_Window window(sx, sy, w, h, "yoshimi start");
+    Fl_Box box(2, 2, w-4, h-4, startup.c_str());
+    box.box(FL_EMBOSSED_FRAME);
+    box.labelsize(16);
+    box.labelfont(FL_BOLD);
+    box.labelcolor(YOSHI_COLOUR);
+    window.end();
+    window.border(false);
+
+    for (int i = 0; i < 10; ++i)
+    {
+        window.show();
+        usleep(10000);
+        Fl::check();
+    }
+}
+#endif
+
 static void *mainGuiThread(void *arg)
 {
-    Fl::lock();
-
     sem_post((sem_t *)arg);
-
     map<SynthEngine *, MusicClient *>::iterator it;
 
+#ifdef GUI_FLTK
+    static bool first = true;
+    if (first)
+    {
+        first = false;
+        Fl::lock();
+    }
     const int textHeight = 15;
     const int textY = 10;
     const unsigned char lred = 0xd7;
@@ -114,7 +183,7 @@ static void *mainGuiThread(void *arg)
     Fl_Window winSplash(splashWidth, splashHeight, "yoshimi splash screen");
     Fl_Box box(0, 0, splashWidth,splashHeight);
     box.image(pix);
-    string startup = YOSHIMI_VERSION;
+    std::string startup = YOSHIMI_VERSION;
     startup = "V " + startup;
     Fl_Box boxLb(0, splashHeight - textY - textHeight, splashWidth, textHeight, startup.c_str());
     boxLb.box(FL_NO_BOX);
@@ -124,7 +193,7 @@ static void *mainGuiThread(void *arg)
     boxLb.labelcolor(fl_rgb_color(lred, lgreen, lblue));
     boxLb.labelfont(FL_HELVETICA | FL_BOLD);
     winSplash.border(false);
-    if (splashSet && bShowGui && firstRuntime->showSplash)
+    if (splashSet && bShowGui && showSplash)
     {
         winSplash.position((Fl::w() - winSplash.w()) / 2, (Fl::h() - winSplash.h()) / 2);
     }
@@ -134,25 +203,34 @@ static void *mainGuiThread(void *arg)
     {
             usleep(33333);
     }
+#endif
     while (firstSynth == NULL); // just wait
 
+#ifdef GUI_FLTK
     GuiThreadMsg::sendMessage(firstSynth, GuiThreadMsg::NewSynthEngine, 0);
-
+#endif
     if (firstRuntime->autoInstance)
-    {
-        for (int i = 1; i < 32; ++i)
-        {
-            if ((firstRuntime->activeInstance >> i) & 1)
-                mainCreateNewInstance(i, true);
-        }
-    }
+        newBlock();
     while (firstRuntime->runSynth)
     {
-        if (firstSynth->getUniqueId() == 0)
+        firstRuntime->signalCheck();
+#ifdef GUI_FLTK
+        if (bShowGui)
         {
-            firstRuntime->signalCheck();
+            if (splashSet)
+            {
+                winSplash.show();
+                usleep(1000);
+                if(time(&here_and_now) < 0) // no time?
+                    here_and_now = old_father_time + SPLASH_TIME;
+                if ((here_and_now - old_father_time) >= SPLASH_TIME)
+                {
+                    splashSet = false;
+                    winSplash.hide();
+                }
+            }
         }
-
+#endif
         for (it = synthInstances.begin(); it != synthInstances.end(); ++it)
         {
             SynthEngine *_synth = it->first;
@@ -160,12 +238,7 @@ static void *mainGuiThread(void *arg)
             if (!_synth->getRuntime().runSynth && _synth->getUniqueId() > 0)
             {
                 if (_synth->getRuntime().configChanged)
-                {
-                    size_t tmpRoot = _synth->ReadBankRoot();
-                    size_t tmpBank = _synth->ReadBank();
-                    _synth->getRuntime().loadConfig(); // restore old settings
-                    _synth->RootBank(tmpRoot, tmpBank); // but keep current root and bank
-                }
+                    _synth->getRuntime().restoreConfig(_synth);
                 _synth->getRuntime().saveConfig();
                 unsigned int instanceID =  _synth->getUniqueId();
                 if (_client)
@@ -185,9 +258,10 @@ static void *mainGuiThread(void *arg)
                 }
 
                 synthInstances.erase(it);
-                cout << "\nStopped " << instanceID << "\n";
+                std::cout << "\nStopped " << instanceID << "\n";
                 break;
             }
+#ifdef GUI_FLTK
             if (bShowGui)
             {
                 for (int i = 0; !_synth->getRuntime().LogList.empty() && i < 5; ++i)
@@ -199,50 +273,42 @@ static void *mainGuiThread(void *arg)
                     }
                 }
             }
-            if (_synth == firstSynth)
-            {
-                int testInstance = startInstance;
-                if (testInstance > 0xff)
-                    startInstance = mainCreateNewInstance(testInstance & 0xff, false);
-            }
+#endif
         }
 
         // where all the action is ...
-        if (bShowGui)
+        if (startInstance > 0x80)
         {
-            if (splashSet)
-            {
-                winSplash.show();
-                usleep(1000);
-                if(time(&here_and_now) < 0) // no time?
-                    here_and_now = old_father_time + SPLASH_TIME;
-                if ((here_and_now - old_father_time) >= SPLASH_TIME)
-                {
-                    splashSet = false;
-                    winSplash.hide();
-                }
-            }
-            Fl::wait(0.033333);
-            GuiThreadMsg::processGuiMessages();
+            int testInstance = startInstance &= 0x7f;
+            configuring = true;
+            mainCreateNewInstance(testInstance);
+            configuring = false;
+            startInstance = testInstance; // to prevent repeats!
         }
         else
-            usleep(33333);
-    }
-    if (firstRuntime->configChanged && (bShowGui | bShowCmdLine)) // don't want this if no cli or gui
-    {
-        size_t tmpRoot = firstSynth->ReadBankRoot();
-        size_t tmpBank = firstSynth->ReadBank();
-        firstRuntime->loadConfig(); // restore old settings
-        firstSynth->RootBank(tmpRoot, tmpBank); // but keep current root and bank
+        {
+#ifdef GUI_FLTK
+            if (bShowGui)
+            {
+                Fl::wait(0.033333);
+                GuiThreadMsg::processGuiMessages();
+            }
+            else
+#endif
+                usleep(33333);
+        }
     }
 
-    firstRuntime->saveConfig();
+    if (firstRuntime->configChanged && (bShowGui | bShowCmdLine)) // don't want this if no cli or gui
+        firstSynth->getRuntime().restoreConfig(firstSynth);
+
+    firstRuntime->saveConfig(true);
     firstSynth->saveHistory();
     firstSynth->saveBanks();
     return NULL;
 }
 
-int mainCreateNewInstance(unsigned int forceId, bool loadState)
+int mainCreateNewInstance(unsigned int forceId)
 {
     MusicClient *musicClient = NULL;
     unsigned int instanceID;
@@ -273,17 +339,7 @@ int mainCreateNewInstance(unsigned int forceId, bool loadState)
         synth->getRuntime().Log("Failed to start MusicIO");
         goto bail_out;
     }
-     // TODO sort this out properly!
-     // it works, but is clunky :(
-    loadState = synth->getRuntime().loadDefaultState;
-    if (loadState)
-    {
-        string name = synth->getRuntime().defaultStateName;
-        if (instanceID > 0)
-            name = name + "-" + to_string(forceId);
-        synth->loadStateAndUpdate(name);
-    }
-
+#ifdef GUI_FLTK
     if (synth->getRuntime().showGui)
     {
         synth->setWindowTitle(musicClient->midiClientName());
@@ -296,18 +352,22 @@ int mainCreateNewInstance(unsigned int forceId, bool loadState)
         if (synth->getRuntime().midiEngine < 1)
             fl_alert("Yoshimi can't find an input system. Running with no MIDI");
     }
-
+    else
+        synth->getRuntime().toConsole = false;
+#else
+    synth->getRuntime().toConsole = false;
+#endif
     synth->getRuntime().StartupReport(musicClient->midiClientName());
-    synth->Unmute();
 
     if (instanceID == 0)
-        cout << "\nYay! We're up and running :-)\n";
+        std::cout << "\nYay! We're up and running :-)\n";
     else
     {
-        cout << "\nStarted "<< instanceID << "\n";
+        std::cout << "\nStarted "<< instanceID << "\n";
         // following copied here for other instances
         synth->installBanks();
     }
+
     synthInstances.insert(std::make_pair(synth, musicClient));
     //register jack ports for enabled parts
     for (int npart = 0; npart < NUM_MIDI_PARTS; ++npart)
@@ -335,30 +395,133 @@ bail_out:
     return -1;
 }
 
-void *commandThread(void *arg = NULL) // silence warning
+void *commandThread(void *) // silence warning (was *arg = NULL)
 {
-    commandInt.cmdIfaceCommandLoop();
+    CmdInterface commandInit;
+    commandInit.cmdIfaceCommandLoop();
     return 0;
 }
 
+std::string runCommand(std::string command, bool clean)
+{
+    const int lineLen = 63;
+    char returnLine[lineLen + 1];
+    FILE *fp = popen(command.c_str(), "r");
+    fgets(returnLine, lineLen, fp);
+    pclose(fp);
+    if (clean)
+    {
+        for (int i = 0; i < lineLen; ++ i)
+        {
+            if (returnLine[i] == ':')
+                returnLine[i] = '0';
+        }
+    }
+    return std::string(returnLine);
+}
+
 int main(int argc, char *argv[])
 {
+    /*
+     * The following is a way to quickly identify and read key config startup values
+     * before the synth engine has started, or any of the normal functions have been
+     * identified. The base config file is quite small and is always uncompressed
+     * (regardless of settings) as it is useful to be able to read and/or manually
+     * change settings under fault conditions.
+     */
+    std::string Home = getenv("HOME");
+    std::string Config = file::loadText(Home + "/.config/yoshimi/yoshimi.config");
+    bool runGui = false;
+    if (Config.empty())
+    {
+        std::cout << "Not there" << std::endl;
+        runGui = true;
+        showSplash = true;
+    }
+    else
+    {
+        int count = 0;
+        int found = 0;
+        while (!Config.empty() && count < 16 && found < 3)
+        { // no need to count beyond 16 lines!
+            std::string line = func::nextLine(Config);
+            //std::cout << count << "  " << line  << std::endl;
+            ++ count;
+
+            if (line.find("enable_gui") != std::string::npos)
+            {
+                ++ found;
+                if (line.find("yes") != std::string::npos)
+                    runGui = true;
+            }
+            if (line.find("enable_splash") != std::string::npos)
+            {
+                ++ found;
+                if (line.find("yes") != std::string::npos)
+                    showSplash = true;
+            }
+            if (line.find("enable_single_master") != std::string::npos)
+            {
+                ++ found;
+                if (line.find("yes") != std::string::npos)
+                    isSingleMaster = true;
+            }
+        }
+    }
+    if (isSingleMaster)
+    {
+        std::string firstText = runCommand("pgrep -o -x yoshimi", false);
+        int firstpid = std::stoi(firstText);
+        int firstTime = std::stoi(runCommand("ps -o etime= -p " + firstText, true));
+        int secondTime = std::stoi(runCommand("ps -o etime= -p " + std::to_string(getpid()), true));
+
+        if ((firstTime - secondTime) > 0)
+        {
+                kill(firstpid, SIGUSR2); // this just sends a message
+                return 0;
+        }
+    }
+#ifdef GUI_FLTK
+    bool useGui = false;
+    if (runGui)
+        useGui = true;
+    if (argc > 1)
+    {
+        for (int n = 1; n < argc; ++ n)
+            if (string(argv[n]) == "-i" || string(argv[n]) == "--no-gui" )
+            {
+                useGui = false; // overRide saved settings
+                break;
+            }
+            else if (string(argv[n]) == "-I" || string(argv[n]) == "--gui" )
+            {
+                useGui = true; // overRide saved settings
+                break;
+            }
+    }
+    if (useGui)
+        do_start();
+
+    bool guiStarted = false;
     time(&old_father_time);
     here_and_now = old_father_time;
+#endif
+
+
     struct termios  oldTerm;
     tcgetattr(0, &oldTerm);
 
-    cout << "Yoshimi " << YOSHIMI_VERSION << " is starting" << endl; // guaranteed start message
+    std::cout << "Yoshimi " << YOSHIMI_VERSION << " is starting" << std::endl; // guaranteed start message
     globalArgc = argc;
     globalArgv = argv;
     bool bExitSuccess = false;
     map<SynthEngine *, MusicClient *>::iterator it;
-    bool guiStarted = false;
+
     pthread_t thr;
     pthread_attr_t attr;
     sem_t semGui;
 
-    if (mainCreateNewInstance(0, false) == -1)
+    if (mainCreateNewInstance(0) == -1)
     {
         goto bail_out;
     }
@@ -367,12 +530,12 @@ int main(int argc, char *argv[])
     firstRuntime = &it->first->getRuntime();
     firstSynth = it->first;
     bShowGui = firstRuntime->showGui;
-    bShowCmdLine = firstRuntime->showCLI;
+    bShowCmdLine = firstRuntime->showCli;
 
     if (firstRuntime->oldConfig)
     {
 
-        cout << "\nExisting config older than " << MIN_CONFIG_MAJOR << "." << MIN_CONFIG_MINOR << "\nCheck settings, save and restart.\n"<< endl;
+        std::cout << "\nExisting config older than " << MIN_CONFIG_MAJOR << "." << MIN_CONFIG_MINOR << "\nCheck settings, save and restart.\n"<< std::endl;
     }
     if(sem_init(&semGui, 0, 0) == 0)
     {
@@ -380,24 +543,28 @@ int main(int argc, char *argv[])
         {
             if (pthread_create(&thr, &attr, mainGuiThread, (void *)&semGui) == 0)
             {
+#ifdef GUI_FLTK
                 guiStarted = true;
+#endif
             }
             pthread_attr_destroy(&attr);
         }
     }
-
+#ifdef GUI_FLTK
     if (!guiStarted)
     {
-        cout << "Yoshimi can't start main gui loop!" << endl;
+        std::cout << "Yoshimi can't start main gui loop!" << std::endl;
         goto bail_out;
     }
     sem_wait(&semGui);
     sem_destroy(&semGui);
-
+#endif
     memset(&yoshimiSigAction, 0, sizeof(yoshimiSigAction));
     yoshimiSigAction.sa_handler = yoshimiSigHandler;
     if (sigaction(SIGUSR1, &yoshimiSigAction, NULL))
         firstRuntime->Log("Setting SIGUSR1 handler failed");
+    if (sigaction(SIGUSR2, &yoshimiSigAction, NULL))
+        firstRuntime->Log("Setting SIGUSR2 handler failed");
     if (sigaction(SIGINT, &yoshimiSigAction, NULL))
         firstRuntime->Log("Setting SIGINT handler failed");
     if (sigaction(SIGHUP, &yoshimiSigAction, NULL))
@@ -419,7 +586,7 @@ int main(int argc, char *argv[])
         {
             if (pthread_create(&cmdThr, &attr, commandThread, (void *)firstSynth) == 0)
             {
-
+                ;
             }
             pthread_attr_destroy(&attr);
         }
@@ -431,7 +598,6 @@ int main(int argc, char *argv[])
     {
         goto bail_out;
     }
-    cout << "\nGoodbye - Play again soon?\n";
     bExitSuccess = true;
 
 bail_out:
@@ -459,9 +625,15 @@ bail_out:
     }
     if(bShowCmdLine)
         tcsetattr(0, TCSANOW, &oldTerm);
-    munlockall(); // just to be sure
     if (bExitSuccess)
-        exit(EXIT_SUCCESS);
+    {
+        int type = firstSynth->getRuntime().exitType;
+        if (type == FORCED_EXIT)
+            std::cout << "\nExit was forced :(" << std::endl;
+        else
+            std::cout << "\nGoodbye - Play again soon?"<< std::endl;
+        exit(type);
+    }
     else
         exit(EXIT_FAILURE);
 }
